@@ -141,7 +141,7 @@ extern int	CONFIG_UNSAFE_USER_PARAMETERS;
 #define ZBX_AVG15		2
 #define ZBX_AVG_COUNT		3
 
-#ifdef _WINDOWS
+#if defined(_WINDOWS)
 #	define MAX_COLLECTOR_PERIOD	(15 * SEC_PER_MIN)
 #endif
 
@@ -192,10 +192,25 @@ int	get_diskstat(const char *devname, zbx_uint64_t *dstat);
 #define PROCESS_MODULE_COMMAND	0x2
 #define PROCESS_WITH_ALIAS	0x4
 
+typedef enum
+{
+	ZBX_KEY_ACCESS_ALLOW,
+	ZBX_KEY_ACCESS_DENY
+}
+zbx_key_access_rule_type_t;
+
 void	init_metrics(void);
 int	add_metric(ZBX_METRIC *metric, char *error, size_t max_error_len);
+int	add_metric_local(ZBX_METRIC *metric, char *error, size_t max_error_len);
 void	free_metrics(void);
 
+void	init_key_access_rules(void);
+void	finalize_key_access_rules_configuration(void);
+int	add_key_access_rule(const char *pattern, zbx_key_access_rule_type_t type);
+int	check_key_access_rules(const char *metric);
+int	check_request_access_rules(AGENT_REQUEST *request);
+void	free_key_access_rules(void);
+
 int	process(const char *in_command, unsigned flags, AGENT_RESULT *result);
 
 int	add_user_parameter(const char *key, char *command, char *error, size_t max_error_len);
@@ -263,13 +278,14 @@ int	SYSTEM_UNAME(AGENT_REQUEST *request, AGENT_RESULT *result);
 int	SYSTEM_BOOTTIME(AGENT_REQUEST *request, AGENT_RESULT *result);
 int	VFS_DEV_READ(AGENT_REQUEST *request, AGENT_RESULT *result);
 int	VFS_DEV_WRITE(AGENT_REQUEST *request, AGENT_RESULT *result);
+int	VFS_DEV_DISCOVERY(AGENT_REQUEST *request, AGENT_RESULT *result);
 int	VFS_FS_INODE(AGENT_REQUEST *request, AGENT_RESULT *result);
 int	VFS_FS_SIZE(AGENT_REQUEST *request, AGENT_RESULT *result);
 int	VFS_FS_DISCOVERY(AGENT_REQUEST *request, AGENT_RESULT *result);
 int	VFS_FS_GET(AGENT_REQUEST *request, AGENT_RESULT *result);
 int	VM_MEMORY_SIZE(AGENT_REQUEST *request, AGENT_RESULT *result);
 
-#ifdef _WINDOWS
+#if defined(_WINDOWS) || defined(__MINGW32__)
 int	USER_PERF_COUNTER(AGENT_REQUEST *request, AGENT_RESULT *result);
 int	PERF_COUNTER(AGENT_REQUEST *request, AGENT_RESULT *result);
 int	PERF_COUNTER_EN(AGENT_REQUEST *request, AGENT_RESULT *result);
@@ -280,6 +296,7 @@ int	SERVICES(AGENT_REQUEST *request, AGENT_RESULT *result);
 int	PROC_INFO(AGENT_REQUEST *request, AGENT_RESULT *result);
 int	NET_IF_LIST(AGENT_REQUEST *request, AGENT_RESULT *result);
 int	WMI_GET(AGENT_REQUEST *request, AGENT_RESULT *result);
+int	WMI_GETALL(AGENT_REQUEST *request, AGENT_RESULT *result);
 int	VM_VMEMORY_SIZE(AGENT_REQUEST *request, AGENT_RESULT *result);
 #endif
 
@@ -287,7 +304,7 @@ int	VM_VMEMORY_SIZE(AGENT_REQUEST *request, AGENT_RESULT *result);
 int	SYSTEM_STAT(AGENT_REQUEST *request, AGENT_RESULT *result);
 #endif
 
-#ifdef _WINDOWS
+#if defined(_WINDOWS) || defined(__MINGW32__)
 typedef int (*zbx_metric_func_t)(AGENT_REQUEST *request, AGENT_RESULT *result, HANDLE timeout_event);
 #else
 typedef int (*zbx_metric_func_t)(AGENT_REQUEST *request, AGENT_RESULT *result);
@@ -344,7 +361,7 @@ void	zbx_mpoints_free(zbx_mpoint_t *mpoint);
 #define ZBX_SYSINFO_PROC_CMDLINE	0x0004
 #define ZBX_SYSINFO_PROC_USER		0x0008
 
-#ifdef _WINDOWS
+#if defined(_WINDOWS) || defined(__MINGW32__)
 #define ZBX_MUTEX_ALL_ALLOW		0
 #define ZBX_MUTEX_THREAD_DENIED		1
 #define ZBX_MUTEX_LOGGING_DENIED	2
