@@ -24,16 +24,16 @@
 #define ZBX_STR(str)	ZBX_STR2(str)
 
 #define APPLICATION_NAME	"Zabbix Agent"
-#define ZABBIX_REVDATE		"23 March 2020"
-#define ZABBIX_VERSION_MAJOR	4
+#define ZABBIX_REVDATE		"11 May 2020"
+#define ZABBIX_VERSION_MAJOR	5
 #define ZABBIX_VERSION_MINOR	0
-#define ZABBIX_VERSION_PATCH	19
+#define ZABBIX_VERSION_PATCH	0
 #ifndef ZABBIX_VERSION_REVISION
-#	define ZABBIX_VERSION_REVISION	a566532084
+#	define ZABBIX_VERSION_REVISION	9665d62db0
 #endif
 #ifdef _WINDOWS
 #	ifndef ZABBIX_VERSION_RC_NUM
-#		define ZABBIX_VERSION_RC_NUM	1400
+#		define ZABBIX_VERSION_RC_NUM	2400
 #	endif
 #endif
 #define ZABBIX_VERSION_RC	""
@@ -41,4 +41,6 @@
 				ZBX_STR(ZABBIX_VERSION_PATCH) ZABBIX_VERSION_RC
 #define ZABBIX_REVISION		ZBX_STR(ZABBIX_VERSION_REVISION)
 
+int	zbx_get_component_version(char *value);
+
 #endif
