@@ -38,6 +38,8 @@ int	zbx_history_record_compare_asc_func(const zbx_history_record_t *d1, const zb
 int	zbx_history_record_compare_desc_func(const zbx_history_record_t *d1, const zbx_history_record_t *d2);
 
 void	zbx_history_value2str(char *buffer, size_t size, const history_value_t *value, int value_type);
+char	*zbx_history_value2str_dyn(const history_value_t *value, int value_type);
+void	zbx_history_value_print(char *buffer, size_t size, const history_value_t *value, int value_type);
 
 /* In most cases zbx_history_record_vector_destroy() function should be used to free the  */
 /* value vector filled by zbx_vc_get_value* functions. This define simply better          */
