@@ -20,6 +20,7 @@
 #include "common.h"
 #include "cfg.h"
 #include "log.h"
+#include "comms.h"
 
 extern unsigned char	program_type;
 
@@ -430,6 +431,13 @@ static int	__parse_cfg_file(const char *cfg_file, struct cfg_line *cfg, int leve
 
 			zabbix_log(LOG_LEVEL_DEBUG, "cfg: para: [%s] val [%s]", parameter, value);
 
+			if (0 == strcmp(parameter, "EnableRemoteCommands"))
+			{
+				zabbix_log(LOG_LEVEL_WARNING, "%s parameter is deprecated,"
+						" use AllowKey=system.run[*] or DenyKey=system.run[*] instead",
+						parameter);
+			}
+
 			if (0 == strcmp(parameter, "Include"))
 			{
 				if (FAIL == parse_cfg_object(value, cfg, level, strict))
@@ -483,6 +491,18 @@ static int	__parse_cfg_file(const char *cfg_file, struct cfg_line *cfg, int leve
 
 						*((zbx_uint64_t *)cfg[i].variable) = var;
 						break;
+					case TYPE_CUSTOM:
+						if (NULL != cfg[i].variable)
+						{
+							cfg_custom_parameter_parser_t custom_parser =
+									(cfg_custom_parameter_parser_t)cfg[i].variable;
+
+							if (SUCCEED != custom_parser(value, &cfg[i]))
+								goto incorrect_config;
+
+							continue;
+						}
+						break;
 					default:
 						assert(0);
 				}
@@ -579,3 +599,48 @@ int	check_cfg_feature_str(const char *parameter, const char *value, const char *
 
 	return SUCCEED;
 }
+
+/******************************************************************************
+ *                                                                            *
+ * Function: zbx_set_data_destination_hosts                                   *
+ *                                                                            *
+ * Purpose: parse "ServerActive' parameter value and set destination servers  *
+ *          using a callback function                                         *
+ *                                                                            *
+ ******************************************************************************/
+void	zbx_set_data_destination_hosts(char *active_hosts, add_serveractive_host_f cb)
+{
+	char	*l = active_hosts, *r;
+
+	do
+	{
+		char		*host = NULL;
+		unsigned short	port;
+
+		if (NULL != (r = strchr(l, ',')))
+			*r = '\0';
+
+		if (SUCCEED != parse_serveractive_element(l, &host, &port, (unsigned short)ZBX_DEFAULT_SERVER_PORT))
+		{
+			zbx_error("error parsing the \"ServerActive\" parameter: address \"%s\" is invalid", l);
+			exit(EXIT_FAILURE);
+		}
+
+		if (SUCCEED != cb(host, port))
+		{
+			zbx_error("error parsing the \"ServerActive\" parameter: address \"%s\" specified more than"
+					" once", l);
+			zbx_free(host);
+			exit(EXIT_FAILURE);
+		}
+
+		zbx_free(host);
+
+		if (NULL != r)
+		{
+			*r = ',';
+			l = r + 1;
+		}
+	}
+	while (NULL != r);
+}
