@@ -21,9 +21,7 @@
 
 #include "zbxdb.h"
 
-#if defined(HAVE_IBM_DB2)
-#	include <sqlcli1.h>
-#elif defined(HAVE_MYSQL)
+#if defined(HAVE_MYSQL)
 #	include "mysql.h"
 #	include "errmsg.h"
 #	include "mysqld_error.h"
@@ -43,14 +41,7 @@
 
 struct zbx_db_result
 {
-#if defined(HAVE_IBM_DB2)
-	SQLHANDLE	hstmt;
-	SQLSMALLINT	nalloc;
-	SQLSMALLINT	ncolumn;
-	DB_ROW		values;
-	DB_ROW		values_cli;
-	SQLINTEGER	*values_len;
-#elif defined(HAVE_MYSQL)
+#if defined(HAVE_MYSQL)
 	MYSQL_RES	*result;
 #elif defined(HAVE_ORACLE)
 	OCIStmt		*stmthp;	/* the statement handle for select operations */
@@ -81,21 +72,7 @@ static char	*last_db_strerror = NULL;	/* last database error message */
 
 extern int	CONFIG_LOG_SLOW_QUERIES;
 
-#if defined(HAVE_IBM_DB2)
-typedef struct
-{
-	SQLHANDLE	henv;
-	SQLHANDLE	hdbc;
-}
-zbx_ibm_db2_handle_t;
-
-static zbx_ibm_db2_handle_t	ibm_db2;
-
-static int	IBM_DB2server_status();
-static int	zbx_ibm_db2_success(SQLRETURN ret);
-static int	zbx_ibm_db2_success_ext(SQLRETURN ret);
-static void	zbx_ibm_db2_log_errors(SQLSMALLINT htype, SQLHANDLE hndl, zbx_err_codes_t err, const char *context);
-#elif defined(HAVE_MYSQL)
+#if defined(HAVE_MYSQL)
 static MYSQL			*conn = NULL;
 #elif defined(HAVE_ORACLE)
 #include "zbxalgo.h"
@@ -331,6 +308,9 @@ static int	is_recoverable_mysql_error(void)
 		case ER_UNKNOWN_COM_ERROR:
 		case ER_LOCK_DEADLOCK:
 		case ER_LOCK_WAIT_TIMEOUT:
+#ifdef CR_SSL_CONNECTION_ERROR
+		case CR_SSL_CONNECTION_ERROR:
+#endif
 #ifdef ER_CONNECTION_KILLED
 		case ER_CONNECTION_KILLED:
 #endif
@@ -363,12 +343,11 @@ static int	is_recoverable_postgresql_error(const PGconn *pg_conn, const PGresult
  *               ZBX_DB_FAIL - failed to connect                              *
  *                                                                            *
  ******************************************************************************/
-int	zbx_db_connect(char *host, char *user, char *password, char *dbname, char *dbschema, char *dbsocket, int port)
+int	zbx_db_connect(char *host, char *user, char *password, char *dbname, char *dbschema, char *dbsocket, int port,
+			char *tls_connect, char *cert, char *key, char *ca, char *cipher, char *cipher_13)
 {
 	int		ret = ZBX_DB_OK, last_txn_error, last_txn_level;
-#if defined(HAVE_IBM_DB2)
-	char		*connect = NULL;
-#elif defined(HAVE_MYSQL)
+#if defined(HAVE_MYSQL)
 #if LIBMYSQL_VERSION_ID >= 80000	/* my_bool type is removed in MySQL 8.0 */
 	bool		mysql_reconnect = 1;
 #else
@@ -379,10 +358,15 @@ int	zbx_db_connect(char *host, char *user, char *password, char *dbname, char *d
 	sword		err = OCI_SUCCESS;
 	static ub2	csid = 0;
 #elif defined(HAVE_POSTGRESQL)
+#	define ZBX_DB_MAX_PARAMS	9
+
 	int		rc;
 	char		*cport = NULL;
 	DB_RESULT	result;
 	DB_ROW		row;
+	const char	*keywords[ZBX_DB_MAX_PARAMS + 1];
+	const char	*values[ZBX_DB_MAX_PARAMS + 1];
+	unsigned int	i = 0;
 #elif defined(HAVE_SQLITE3)
 	char		*p, *path = NULL;
 #endif
@@ -400,88 +384,7 @@ int	zbx_db_connect(char *host, char *user, char *password, char *dbname, char *d
 	txn_error = ZBX_DB_OK;
 	txn_level = 0;
 
-#if defined(HAVE_IBM_DB2)
-	connect = zbx_strdup(connect, "PROTOCOL=TCPIP;");
-	if ('\0' != *host)
-		connect = zbx_strdcatf(connect, "HOSTNAME=%s;", host);
-	if (NULL != dbname && '\0' != *dbname)
-		connect = zbx_strdcatf(connect, "DATABASE=%s;", dbname);
-	if (0 != port)
-		connect = zbx_strdcatf(connect, "PORT=%d;", port);
-	if (NULL != user && '\0' != *user)
-		connect = zbx_strdcatf(connect, "UID=%s;", user);
-	if (NULL != password && '\0' != *password)
-		connect = zbx_strdcatf(connect, "PWD=%s;", password);
-
-	memset(&ibm_db2, 0, sizeof(ibm_db2));
-
-	/* allocate an environment handle */
-	if (SUCCEED != zbx_ibm_db2_success(SQLAllocHandle(SQL_HANDLE_ENV, SQL_NULL_HANDLE, &ibm_db2.henv)))
-		ret = ZBX_DB_FAIL;
-
-	/* set attribute to enable application to run as ODBC 3.0 application; */
-	/* recommended for pure IBM DB2 CLI, but not required */
-	if (ZBX_DB_OK == ret && SUCCEED != zbx_ibm_db2_success(SQLSetEnvAttr(ibm_db2.henv, SQL_ATTR_ODBC_VERSION,
-			(void *)SQL_OV_ODBC3, 0)))
-	{
-		ret = ZBX_DB_FAIL;
-	}
-
-	/* allocate a database connection handle */
-	if (ZBX_DB_OK == ret && SUCCEED != zbx_ibm_db2_success(SQLAllocHandle(SQL_HANDLE_DBC, ibm_db2.henv,
-			&ibm_db2.hdbc)))
-	{
-		ret = ZBX_DB_FAIL;
-	}
-
-	/* set codepage to utf-8 */
-	if (ZBX_DB_OK == ret && SUCCEED != zbx_ibm_db2_success(SQLSetConnectAttr(ibm_db2.hdbc, SQL_ATTR_CLIENT_CODEPAGE,
-			(SQLPOINTER)(SQLUINTEGER)1208, SQL_IS_UINTEGER)))
-	{
-		ret = ZBX_DB_FAIL;
-	}
-
-	/* connect to the database */
-	if (ZBX_DB_OK == ret && SUCCEED != zbx_ibm_db2_success(SQLDriverConnect(ibm_db2.hdbc, NULL, (SQLCHAR *)connect,
-			SQL_NTS, NULL, 0, NULL, SQL_DRIVER_NOPROMPT)))
-	{
-		ret = ZBX_DB_FAIL;
-	}
-
-	/* set autocommit on */
-	if (ZBX_DB_OK == ret && SUCCEED != zbx_ibm_db2_success(SQLSetConnectAttr(ibm_db2.hdbc, SQL_ATTR_AUTOCOMMIT,
-			(SQLPOINTER)SQL_AUTOCOMMIT_ON, SQL_NTS)))
-	{
-		ret = ZBX_DB_DOWN;
-	}
-
-	/* we do not generate vendor escape clause sequences */
-	if (ZBX_DB_OK == ret && SUCCEED != zbx_ibm_db2_success(SQLSetConnectAttr(ibm_db2.hdbc, SQL_ATTR_NOSCAN,
-			(SQLPOINTER)SQL_NOSCAN_ON, SQL_NTS)))
-	{
-		ret = ZBX_DB_DOWN;
-	}
-
-	/* set current schema */
-	if (NULL != dbschema && '\0' != *dbschema && ZBX_DB_OK == ret)
-	{
-		char	*dbschema_esc;
-
-		dbschema_esc = zbx_db_dyn_escape_string(dbschema, ZBX_SIZE_T_MAX, ZBX_SIZE_T_MAX, ESCAPE_SEQUENCE_ON);
-		if (0 < (ret = zbx_db_execute("set current schema='%s'", dbschema_esc)))
-			ret = ZBX_DB_OK;
-		zbx_free(dbschema_esc);
-	}
-
-	zbx_free(connect);
-
-	/* output error information */
-	if (ZBX_DB_OK != ret)
-	{
-		zbx_ibm_db2_log_errors(SQL_HANDLE_ENV, ibm_db2.henv, ERR_Z3001, dbname);
-		zbx_ibm_db2_log_errors(SQL_HANDLE_DBC, ibm_db2.hdbc, ERR_Z3001, dbname);
-	}
-#elif defined(HAVE_MYSQL)
+#if defined(HAVE_MYSQL)
 	ZBX_UNUSED(dbschema);
 
 	if (NULL == (conn = mysql_init(NULL)))
@@ -489,8 +392,118 @@ int	zbx_db_connect(char *host, char *user, char *password, char *dbname, char *d
 		zabbix_log(LOG_LEVEL_CRIT, "cannot allocate or initialize MYSQL database connection object");
 		exit(EXIT_FAILURE);
 	}
+#if defined(HAVE_MYSQL_TLS)
+	if (NULL != tls_connect)
+	{
+		unsigned int	mysql_tls_mode;
 
-	if (NULL == mysql_real_connect(conn, host, user, password, dbname, port, dbsocket, CLIENT_MULTI_STATEMENTS))
+		if (0 == strcmp(tls_connect, ZBX_DB_TLS_CONNECT_REQUIRED_TXT))
+			mysql_tls_mode = SSL_MODE_REQUIRED;
+		else if (0 == strcmp(tls_connect, ZBX_DB_TLS_CONNECT_VERIFY_CA_TXT))
+			mysql_tls_mode = SSL_MODE_VERIFY_CA;
+		else
+			mysql_tls_mode = SSL_MODE_VERIFY_IDENTITY;
+
+		if (0 != mysql_options(conn, MYSQL_OPT_SSL_MODE, &mysql_tls_mode))
+		{
+			zabbix_log(LOG_LEVEL_ERR, "Cannot set MYSQL_OPT_SSL_MODE option.");
+			ret = ZBX_DB_FAIL;
+		}
+	}
+
+	if (ZBX_DB_OK == ret && NULL != ca && 0 != mysql_options(conn, MYSQL_OPT_SSL_CA, ca))
+	{
+		zabbix_log(LOG_LEVEL_ERR, "Cannot set MYSQL_OPT_SSL_CA option.");
+		ret = ZBX_DB_FAIL;
+	}
+
+	if (ZBX_DB_OK == ret && NULL != key && 0 != mysql_options(conn, MYSQL_OPT_SSL_KEY, key))
+	{
+		zabbix_log(LOG_LEVEL_ERR, "Cannot set MYSQL_OPT_SSL_KEY option.");
+		ret = ZBX_DB_FAIL;
+	}
+
+	if (ZBX_DB_OK == ret && NULL != cert && 0 != mysql_options(conn, MYSQL_OPT_SSL_CERT, cert))
+	{
+		zabbix_log(LOG_LEVEL_ERR, "Cannot set MYSQL_OPT_SSL_CERT option.");
+		ret = ZBX_DB_FAIL;
+	}
+
+	if (ZBX_DB_OK == ret && NULL != cipher && 0 != mysql_options(conn, MYSQL_OPT_SSL_CIPHER, cipher))
+	{
+		zabbix_log(LOG_LEVEL_ERR, "Cannot set MYSQL_OPT_SSL_CIPHER option.");
+		ret = ZBX_DB_FAIL;
+	}
+#if defined(HAVE_MYSQL_TLS_CIPHERSUITES)
+	if (ZBX_DB_OK == ret && NULL != cipher_13 && 0 != mysql_options(conn, MYSQL_OPT_TLS_CIPHERSUITES, cipher_13))
+	{
+		zabbix_log(LOG_LEVEL_ERR, "Cannot set MYSQL_OPT_TLS_CIPHERSUITES option.");
+		ret = ZBX_DB_FAIL;
+	}
+#else
+	ZBX_UNUSED(cipher_13);
+#endif
+#elif defined(HAVE_MARIADB_TLS)
+	ZBX_UNUSED(cipher_13);
+
+	if (NULL != tls_connect)
+	{
+		if (0 == strcmp(tls_connect, ZBX_DB_TLS_CONNECT_REQUIRED_TXT))
+		{
+			my_bool	enforce_tls = 1;
+
+			if (0 != mysql_optionsv(conn, MYSQL_OPT_SSL_ENFORCE, (void *)&enforce_tls))
+			{
+				zabbix_log(LOG_LEVEL_ERR, "Cannot set MYSQL_OPT_SSL_ENFORCE option.");
+				ret = ZBX_DB_FAIL;
+			}
+		}
+		else
+		{
+			my_bool	verify = 1;
+
+			if (0 != mysql_optionsv(conn, MYSQL_OPT_SSL_VERIFY_SERVER_CERT, (void *)&verify))
+			{
+				zabbix_log(LOG_LEVEL_ERR, "Cannot set MYSQL_OPT_SSL_VERIFY_SERVER_CERT option.");
+				ret = ZBX_DB_FAIL;
+			}
+		}
+	}
+
+	if (ZBX_DB_OK == ret && NULL != ca && 0 != mysql_optionsv(conn, MYSQL_OPT_SSL_CA, ca))
+	{
+		zabbix_log(LOG_LEVEL_ERR, "Cannot set MYSQL_OPT_SSL_CA option.");
+		ret = ZBX_DB_FAIL;
+	}
+
+	if (ZBX_DB_OK == ret && NULL != key && 0 != mysql_optionsv(conn, MYSQL_OPT_SSL_KEY, key))
+	{
+		zabbix_log(LOG_LEVEL_ERR, "Cannot set MYSQL_OPT_SSL_KEY option.");
+		ret = ZBX_DB_FAIL;
+	}
+
+	if (ZBX_DB_OK == ret && NULL != cert && 0 != mysql_optionsv(conn, MYSQL_OPT_SSL_CERT, cert))
+	{
+		zabbix_log(LOG_LEVEL_ERR, "Cannot set MYSQL_OPT_SSL_CERT option.");
+		ret = ZBX_DB_FAIL;
+	}
+
+	if (ZBX_DB_OK == ret && NULL != cipher && 0 != mysql_optionsv(conn, MYSQL_OPT_SSL_CIPHER, cipher))
+	{
+		zabbix_log(LOG_LEVEL_ERR, "Cannot set MYSQL_OPT_SSL_CIPHER option.");
+		ret = ZBX_DB_FAIL;
+	}
+#else
+	ZBX_UNUSED(tls_connect);
+	ZBX_UNUSED(cert);
+	ZBX_UNUSED(key);
+	ZBX_UNUSED(ca);
+	ZBX_UNUSED(cipher);
+	ZBX_UNUSED(cipher_13);
+#endif
+	if (ZBX_DB_OK == ret &&
+			NULL == mysql_real_connect(conn, host, user, password, dbname, port, dbsocket,
+				CLIENT_MULTI_STATEMENTS))
 	{
 		zbx_db_errlog(ERR_Z3001, mysql_errno(conn), mysql_error(conn), dbname);
 		ret = ZBX_DB_FAIL;
@@ -523,9 +536,14 @@ int	zbx_db_connect(char *host, char *user, char *password, char *dbname, char *d
 
 	if (ZBX_DB_FAIL == ret && SUCCEED == is_recoverable_mysql_error())
 		ret = ZBX_DB_DOWN;
-
 #elif defined(HAVE_ORACLE)
 	ZBX_UNUSED(dbschema);
+	ZBX_UNUSED(tls_connect);
+	ZBX_UNUSED(cert);
+	ZBX_UNUSED(key);
+	ZBX_UNUSED(ca);
+	ZBX_UNUSED(cipher);
+	ZBX_UNUSED(cipher_13);
 
 	memset(&oracle, 0, sizeof(oracle));
 
@@ -591,6 +609,7 @@ int	zbx_db_connect(char *host, char *user, char *password, char *dbname, char *d
 			case OCI_SUCCESS_WITH_INFO:
 				zabbix_log(LOG_LEVEL_WARNING, "%s", zbx_oci_error(err, NULL));
 				/* break; is not missing here */
+				ZBX_FALLTHROUGH;
 			case OCI_SUCCESS:
 				err = OCIAttrGet((void *)oracle.svchp, OCI_HTYPE_SVCCTX, (void *)&oracle.srvhp,
 						(ub4 *)0, OCI_ATTR_SERVER, oracle.errhp);
@@ -624,10 +643,73 @@ int	zbx_db_connect(char *host, char *user, char *password, char *dbname, char *d
 
 	zbx_free(connect);
 #elif defined(HAVE_POSTGRESQL)
-	if (0 != port)
-		cport = zbx_dsprintf(cport, "%d", port);
+	ZBX_UNUSED(cipher);
+	ZBX_UNUSED(cipher_13);
 
-	conn = PQsetdbLogin(host, cport, NULL, NULL, dbname, user, password);
+	if (NULL != tls_connect)
+	{
+		keywords[i] = "sslmode";
+
+		if (0 == strcmp(tls_connect, ZBX_DB_TLS_CONNECT_REQUIRED_TXT))
+			values[i++] = "require";
+		else if (0 == strcmp(tls_connect, ZBX_DB_TLS_CONNECT_VERIFY_CA_TXT))
+			values[i++] = "verify-ca";
+		else
+			values[i++] = "verify-full";
+	}
+
+	if (NULL != cert)
+	{
+		keywords[i] = "sslcert";
+		values[i++] = cert;
+	}
+
+	if (NULL != key)
+	{
+		keywords[i] = "sslkey";
+		values[i++] = key;
+	}
+
+	if (NULL != ca)
+	{
+		keywords[i] = "sslrootcert";
+		values[i++] = ca;
+	}
+
+	if (NULL != host)
+	{
+		keywords[i] = "host";
+		values[i++] = host;
+	}
+
+	if (NULL != dbname)
+	{
+		keywords[i] = "dbname";
+		values[i++] = dbname;
+	}
+
+	if (NULL != user)
+	{
+		keywords[i] = "user";
+		values[i++] = user;
+	}
+
+	if (NULL != password)
+	{
+		keywords[i] = "password";
+		values[i++] = password;
+	}
+
+	if (0 != port)
+	{
+		keywords[i] = "port";
+		values[i++] = cport = zbx_dsprintf(cport, "%d", port);
+	}
+
+	keywords[i] = NULL;
+	values[i] = NULL;
+
+	conn = PQconnectdbParams(keywords, values, 0);
 
 	zbx_free(cport);
 
@@ -674,6 +756,13 @@ int	zbx_db_connect(char *host, char *user, char *password, char *dbname, char *d
 	if (ZBX_DB_OK != ret)
 		goto out;
 
+	/* increase float precision */
+	if (0 < (ret = zbx_db_execute("set extra_float_digits to 3")))
+		ret = ZBX_DB_OK;
+
+	if (ZBX_DB_OK != ret)
+		goto out;
+
 	result = zbx_db_select("show standard_conforming_strings");
 
 	if ((DB_RESULT)ZBX_DB_DOWN == result || NULL == result)
@@ -699,6 +788,12 @@ out:
 	ZBX_UNUSED(password);
 	ZBX_UNUSED(dbschema);
 	ZBX_UNUSED(port);
+	ZBX_UNUSED(tls_connect);
+	ZBX_UNUSED(cert);
+	ZBX_UNUSED(key);
+	ZBX_UNUSED(ca);
+	ZBX_UNUSED(cipher);
+	ZBX_UNUSED(cipher_13);
 #ifdef HAVE_FUNCTION_SQLITE3_OPEN_V2
 	if (SQLITE_OK != sqlite3_open_v2(dbname, &conn, SQLITE_OPEN_READWRITE, NULL))
 #else
@@ -791,18 +886,7 @@ void	zbx_db_deinit(void)
 
 void	zbx_db_close(void)
 {
-#if defined(HAVE_IBM_DB2)
-	if (ibm_db2.hdbc)
-	{
-		SQLDisconnect(ibm_db2.hdbc);
-		SQLFreeHandle(SQL_HANDLE_DBC, ibm_db2.hdbc);
-	}
-
-	if (ibm_db2.henv)
-		SQLFreeHandle(SQL_HANDLE_ENV, ibm_db2.henv);
-
-	memset(&ibm_db2, 0, sizeof(ibm_db2));
-#elif defined(HAVE_MYSQL)
+#if defined(HAVE_MYSQL)
 	if (NULL != conn)
 	{
 		mysql_close(conn);
@@ -891,27 +975,7 @@ int	zbx_db_begin(void)
 
 	txn_level++;
 
-#if defined(HAVE_IBM_DB2)
-	if (SUCCEED != zbx_ibm_db2_success(SQLSetConnectAttr(ibm_db2.hdbc, SQL_ATTR_AUTOCOMMIT,
-			(SQLPOINTER)SQL_AUTOCOMMIT_OFF, SQL_NTS)))
-	{
-		rc = ZBX_DB_DOWN;
-	}
-
-	if (ZBX_DB_OK == rc)
-	{
-		/* create savepoint for correct rollback on DB2 */
-		if (0 <= (rc = zbx_db_execute("savepoint zbx_begin_savepoint unique on rollback retain cursors;")))
-			rc = ZBX_DB_OK;
-	}
-
-	if (ZBX_DB_OK != rc)
-	{
-		zbx_ibm_db2_log_errors(SQL_HANDLE_DBC, ibm_db2.hdbc, ERR_Z3005, "<begin>");
-		rc = (SQL_CD_TRUE == IBM_DB2server_status() ? ZBX_DB_FAIL : ZBX_DB_DOWN);
-	}
-
-#elif defined(HAVE_MYSQL) || defined(HAVE_POSTGRESQL)
+#if defined(HAVE_MYSQL) || defined(HAVE_POSTGRESQL)
 	rc = zbx_db_execute("begin;");
 #elif defined(HAVE_SQLITE3)
 	zbx_mutex_lock(sqlite_access);
@@ -950,20 +1014,7 @@ int	zbx_db_commit(void)
 	if (ZBX_DB_OK != txn_error)
 		return ZBX_DB_FAIL; /* commit called on failed transaction */
 
-#if defined(HAVE_IBM_DB2)
-	if (SUCCEED != zbx_ibm_db2_success(SQLEndTran(SQL_HANDLE_DBC, ibm_db2.hdbc, SQL_COMMIT)))
-		rc = ZBX_DB_DOWN;
-	if (SUCCEED != zbx_ibm_db2_success(SQLSetConnectAttr(ibm_db2.hdbc, SQL_ATTR_AUTOCOMMIT,
-			(SQLPOINTER)SQL_AUTOCOMMIT_ON, SQL_NTS)))
-	{
-		rc = ZBX_DB_DOWN;
-	}
-	if (ZBX_DB_OK != rc)
-	{
-		zbx_ibm_db2_log_errors(SQL_HANDLE_DBC, ibm_db2.hdbc, ERR_Z3005, "<commit>");
-		rc = (SQL_CD_TRUE == IBM_DB2server_status() ? ZBX_DB_FAIL : ZBX_DB_DOWN);
-	}
-#elif defined(HAVE_ORACLE)
+#if defined(HAVE_ORACLE)
 	if (OCI_SUCCESS != (err = OCITransCommit(oracle.svchp, oracle.errhp, OCI_DEFAULT)))
 		rc = OCI_handle_sql_error(ERR_Z3005, err, "commit failed");
 #elif defined(HAVE_MYSQL) || defined(HAVE_POSTGRESQL) || defined(HAVE_SQLITE3)
@@ -1013,24 +1064,7 @@ int	zbx_db_rollback(void)
 	/* allow rollback of failed transaction */
 	txn_error = ZBX_DB_OK;
 
-#if defined(HAVE_IBM_DB2)
-
-	/* Rollback to begin that is marked with savepoint. This move undo all transactions. */
-	if (0 <= (rc = zbx_db_execute("rollback to savepoint zbx_begin_savepoint;")))
-		rc = ZBX_DB_OK;
-
-	if (SUCCEED != zbx_ibm_db2_success(SQLSetConnectAttr(ibm_db2.hdbc, SQL_ATTR_AUTOCOMMIT,
-			(SQLPOINTER)SQL_AUTOCOMMIT_ON, SQL_NTS)))
-	{
-		rc = ZBX_DB_DOWN;
-	}
-
-	if (ZBX_DB_OK != rc)
-	{
-		zbx_ibm_db2_log_errors(SQL_HANDLE_DBC, ibm_db2.hdbc, ERR_Z3005, "<rollback>");
-		rc = (SQL_CD_TRUE == IBM_DB2server_status() ? ZBX_DB_FAIL : ZBX_DB_DOWN);
-	}
-#elif defined(HAVE_MYSQL) || defined(HAVE_POSTGRESQL)
+#if defined(HAVE_MYSQL) || defined(HAVE_POSTGRESQL)
 	rc = zbx_db_execute("rollback;");
 #elif defined(HAVE_ORACLE)
 	if (OCI_SUCCESS != (err = OCITransRollback(oracle.svchp, oracle.errhp, OCI_DEFAULT)))
@@ -1143,7 +1177,7 @@ static sb4 db_bind_dynamic_cb(dvoid *ctxp, OCIBind *bindp, ub4 iter, ub4 index,
 				*alenp = 0;
 				break;
 			}
-			/* break; is not missing here */
+			ZBX_FALLTHROUGH;
 		case ZBX_TYPE_UINT:
 			*bufpp = &((OCINumber *)context->data)[iter];
 			*alenp = sizeof(OCINumber);
@@ -1230,7 +1264,7 @@ int	zbx_db_bind_parameter_dyn(zbx_db_bind_context_t *context, int position, unsi
 			break;
 		case ZBX_TYPE_FLOAT:
 			context->size_max = sizeof(double);
-			data_type = SQLT_FLT;
+			data_type = SQLT_BDOUBLE;
 			break;
 		case ZBX_TYPE_CHAR:
 		case ZBX_TYPE_TEXT:
@@ -1299,8 +1333,6 @@ void	zbx_db_clean_bind_context(zbx_db_bind_context_t *context)
 
 int	zbx_db_statement_execute(int iters)
 {
-	const char	*__function_name = "zbx_db_statement_execute";
-
 	sword	err;
 	ub4	nrows;
 	int	ret;
@@ -1323,7 +1355,7 @@ int	zbx_db_statement_execute(int iters)
 		txn_error = ZBX_DB_FAIL;
 	}
 out:
-	zabbix_log(LOG_LEVEL_DEBUG, "%s():%d", __function_name, ret);
+	zabbix_log(LOG_LEVEL_DEBUG, "%s():%d", __func__, ret);
 
 	return ret;
 }
@@ -1344,12 +1376,7 @@ int	zbx_db_vexecute(const char *fmt, va_list args)
 	char	*sql = NULL;
 	int	ret = ZBX_DB_OK;
 	double	sec = 0;
-#if defined(HAVE_IBM_DB2)
-	SQLHANDLE	hstmt = 0;
-	SQLRETURN	ret1;
-	SQLLEN		row1;
-	SQLLEN		rows = 0;
-#elif defined(HAVE_MYSQL)
+#if defined(HAVE_MYSQL)
 	int		status;
 #elif defined(HAVE_ORACLE)
 	sword		err = OCI_SUCCESS;
@@ -1378,46 +1405,7 @@ int	zbx_db_vexecute(const char *fmt, va_list args)
 
 	zabbix_log(LOG_LEVEL_DEBUG, "query [txnlev:%d] [%s]", txn_level, sql);
 
-#if defined(HAVE_IBM_DB2)
-	/* allocate a statement handle */
-	if (SUCCEED != zbx_ibm_db2_success(SQLAllocHandle(SQL_HANDLE_STMT, ibm_db2.hdbc, &hstmt)))
-		ret = ZBX_DB_DOWN;
-
-	/* directly execute the statement; returns SQL_NO_DATA_FOUND when no rows were affected */
-  	if (ZBX_DB_OK == ret && SUCCEED != zbx_ibm_db2_success_ext(SQLExecDirect(hstmt, (SQLCHAR *)sql, SQL_NTS)))
-		ret = ZBX_DB_DOWN;
-
-	/* get number of affected rows */
-	if (ZBX_DB_OK == ret && SUCCEED != zbx_ibm_db2_success(SQLRowCount(hstmt, &rows)))
-		ret = ZBX_DB_DOWN;
-
-	/* process other SQL statements in the batch */
-	while (ZBX_DB_OK == ret && SUCCEED == zbx_ibm_db2_success(ret1 = SQLMoreResults(hstmt)))
-	{
-		if (SUCCEED != zbx_ibm_db2_success(SQLRowCount(hstmt, &row1)))
-			ret = ZBX_DB_DOWN;
-		else
-			rows += row1;
-	}
-
-	if (ZBX_DB_OK == ret && SQL_NO_DATA_FOUND != ret1)
-		ret = ZBX_DB_DOWN;
-
-	if (ZBX_DB_OK != ret)
-	{
-		zbx_ibm_db2_log_errors(SQL_HANDLE_DBC, ibm_db2.hdbc, ERR_Z3005, sql);
-		zbx_ibm_db2_log_errors(SQL_HANDLE_STMT, hstmt, ERR_Z3005, sql);
-
-		ret = (SQL_CD_TRUE == IBM_DB2server_status() ? ZBX_DB_FAIL : ZBX_DB_DOWN);
-	}
-	else if (0 <= rows)
-	{
-		ret = (int)rows;
-	}
-
-	if (hstmt)
-		SQLFreeHandle(SQL_HANDLE_STMT, hstmt);
-#elif defined(HAVE_MYSQL)
+#if defined(HAVE_MYSQL)
 	if (NULL == conn)
 	{
 		zbx_db_errlog(ERR_Z3003, 0, NULL, NULL);
@@ -1554,10 +1542,7 @@ DB_RESULT	zbx_db_vselect(const char *fmt, va_list args)
 	char		*sql = NULL;
 	DB_RESULT	result = NULL;
 	double		sec = 0;
-#if defined(HAVE_IBM_DB2)
-	int		i;
-	SQLRETURN	ret = SQL_SUCCESS;
-#elif defined(HAVE_ORACLE)
+#if defined(HAVE_ORACLE)
 	sword		err = OCI_SUCCESS;
 	ub4		prefetch_rows = 200, counter;
 #elif defined(HAVE_POSTGRESQL)
@@ -1580,63 +1565,7 @@ DB_RESULT	zbx_db_vselect(const char *fmt, va_list args)
 
 	zabbix_log(LOG_LEVEL_DEBUG, "query [txnlev:%d] [%s]", txn_level, sql);
 
-#if defined(HAVE_IBM_DB2)
-	result = zbx_malloc(result, sizeof(struct zbx_db_result));
-	memset(result, 0, sizeof(struct zbx_db_result));
-
-	/* allocate a statement handle */
-	if (SUCCEED != zbx_ibm_db2_success(ret = SQLAllocHandle(SQL_HANDLE_STMT, ibm_db2.hdbc, &result->hstmt)))
-		goto error;
-
-	/* directly execute the statement */
-	if (SUCCEED != zbx_ibm_db2_success(ret = SQLExecDirect(result->hstmt, (SQLCHAR *)sql, SQL_NTS)))
-		goto error;
-
-	/* identify the number of output columns */
-	if (SUCCEED != zbx_ibm_db2_success(ret = SQLNumResultCols(result->hstmt, &result->ncolumn)))
-		goto error;
-
-	if (0 == result->ncolumn)
-		goto error;
-
-	result->nalloc = 0;
-	result->values = zbx_malloc(result->values, sizeof(char *) * result->ncolumn);
-	result->values_cli = zbx_malloc(result->values_cli, sizeof(char *) * result->ncolumn);
-	result->values_len = zbx_malloc(result->values_len, sizeof(SQLINTEGER) * result->ncolumn);
-
-	for (i = 0; i < result->ncolumn; i++)
-	{
-		/* get the display size for a column */
-		if (SUCCEED != zbx_ibm_db2_success(ret = SQLColAttribute(result->hstmt, (SQLSMALLINT)(i + 1),
-				SQL_DESC_DISPLAY_SIZE, NULL, 0, NULL, &result->values_len[i])))
-		{
-			goto error;
-		}
-
-		result->values_len[i] += 1; /* '\0'; */
-
-		/* allocate memory to bind a column */
-		result->values_cli[i] = zbx_malloc(NULL, result->values_len[i]);
-		result->nalloc++;
-
-		/* bind columns to program variables, converting all types to CHAR */
-		if (SUCCEED != zbx_ibm_db2_success(ret = SQLBindCol(result->hstmt, (SQLSMALLINT)(i + 1),
-				SQL_C_CHAR, result->values_cli[i], result->values_len[i], &result->values_len[i])))
-		{
-			goto error;
-		}
-	}
-error:
-	if (SUCCEED != zbx_ibm_db2_success(ret) || 0 == result->ncolumn)
-	{
-		zbx_ibm_db2_log_errors(SQL_HANDLE_DBC, ibm_db2.hdbc, ERR_Z3005, sql);
-		zbx_ibm_db2_log_errors(SQL_HANDLE_STMT, result->hstmt, ERR_Z3005, sql);
-
-		DBfree_result(result);
-
-		result = (SQL_CD_TRUE == IBM_DB2server_status() ? NULL : (DB_RESULT)ZBX_DB_DOWN);
-	}
-#elif defined(HAVE_MYSQL)
+#if defined(HAVE_MYSQL)
 	result = (DB_RESULT)zbx_malloc(NULL, sizeof(struct zbx_db_result));
 	result->result = NULL;
 
@@ -1748,33 +1677,38 @@ error:
 		}
 		else
 		{
-			if (OCI_SUCCESS == err)
+			if (SQLT_IBDOUBLE != data_type && SQLT_BDOUBLE != data_type)
 			{
-				/* retrieve the length semantics for the column */
-				char_semantics = 0;
-				err = OCIAttrGet((void *)parmdp, (ub4)OCI_DTYPE_PARAM, (void *)&char_semantics,
-						(ub4 *)NULL, (ub4)OCI_ATTR_CHAR_USED, (OCIError *)oracle.errhp);
-			}
-
-			if (OCI_SUCCESS == err)
-			{
-				if (0 != char_semantics)
+				if (OCI_SUCCESS == err)
 				{
-					/* retrieve the column width in characters */
-					err = OCIAttrGet((void *)parmdp, (ub4)OCI_DTYPE_PARAM, (void *)&col_width,
-							(ub4 *)NULL, (ub4)OCI_ATTR_CHAR_SIZE, (OCIError *)oracle.errhp);
+					/* retrieve the length semantics for the column */
+					char_semantics = 0;
+					err = OCIAttrGet((void *)parmdp, (ub4)OCI_DTYPE_PARAM, (void *)&char_semantics,
+							(ub4 *)NULL, (ub4)OCI_ATTR_CHAR_USED, (OCIError *)oracle.errhp);
+				}
 
-					/* adjust for UTF-8 */
-					col_width *= 4;
-				}
-				else
+				if (OCI_SUCCESS == err)
 				{
-					/* retrieve the column width in bytes */
-					err = OCIAttrGet((void *)parmdp, (ub4)OCI_DTYPE_PARAM, (void *)&col_width,
-							(ub4 *)NULL, (ub4)OCI_ATTR_DATA_SIZE, (OCIError *)oracle.errhp);
+					if (0 != char_semantics)
+					{
+						/* retrieve the column width in characters */
+						err = OCIAttrGet((void *)parmdp, (ub4)OCI_DTYPE_PARAM, (void *)&col_width,
+								(ub4 *)NULL, (ub4)OCI_ATTR_CHAR_SIZE, (OCIError *)oracle.errhp);
+
+						/* adjust for UTF-8 */
+						col_width *= 4;
+					}
+					else
+					{
+						/* retrieve the column width in bytes */
+						err = OCIAttrGet((void *)parmdp, (ub4)OCI_DTYPE_PARAM, (void *)&col_width,
+								(ub4 *)NULL, (ub4)OCI_ATTR_DATA_SIZE, (OCIError *)oracle.errhp);
+					}
 				}
+				col_width++;	/* add 1 byte for terminating '\0' */
 			}
-			col_width++;	/* add 1 byte for terminating '\0' */
+			else
+				col_width = ZBX_MAX_DOUBLE_LEN + 1;
 
 			result->values_alloc[counter - 1] = col_width;
 			result->values[counter - 1] = zbx_malloc(NULL, col_width);
@@ -1890,15 +1824,9 @@ clean:
  */
 DB_RESULT	zbx_db_select_n(const char *query, int n)
 {
-#if defined(HAVE_IBM_DB2)
-	return zbx_db_select("%s fetch first %d rows only", query, n);
-#elif defined(HAVE_MYSQL)
-	return zbx_db_select("%s limit %d", query, n);
-#elif defined(HAVE_ORACLE)
+#if defined(HAVE_ORACLE)
 	return zbx_db_select("select * from (%s) where rownum<=%d", query, n);
-#elif defined(HAVE_POSTGRESQL)
-	return zbx_db_select("%s limit %d", query, n);
-#elif defined(HAVE_SQLITE3)
+#elif defined(HAVE_MYSQL) || defined(HAVE_POSTGRESQL) || defined(HAVE_SQLITE3)
 	return zbx_db_select("%s limit %d", query, n);
 #endif
 }
@@ -1957,9 +1885,7 @@ static size_t	zbx_db_bytea_unescape(u_char *io)
 
 DB_ROW	zbx_db_fetch(DB_RESULT result)
 {
-#if defined(HAVE_IBM_DB2)
-	int		i;
-#elif defined(HAVE_ORACLE)
+#if defined(HAVE_ORACLE)
 	int		i;
 	sword		rc;
 	static char	errbuf[512];
@@ -1969,15 +1895,7 @@ DB_ROW	zbx_db_fetch(DB_RESULT result)
 	if (NULL == result)
 		return NULL;
 
-#if defined(HAVE_IBM_DB2)
-	if (SUCCEED != zbx_ibm_db2_success(SQLFetch(result->hstmt)))	/* e.g., SQL_NO_DATA_FOUND */
-		return NULL;
-
-	for (i = 0; i < result->ncolumn; i++)
-		result->values[i] = (SQL_NULL_DATA == result->values_len[i] ? NULL : result->values_cli[i]);
-
-	return result->values;
-#elif defined(HAVE_MYSQL)
+#if defined(HAVE_MYSQL)
 	if (NULL == result->result)
 		return NULL;
 
@@ -2167,29 +2085,7 @@ static void	OCI_DBclean_result(DB_RESULT result)
 
 void	DBfree_result(DB_RESULT result)
 {
-#if defined(HAVE_IBM_DB2)
-	if (NULL == result)
-		return;
-
-	if (NULL != result->values_cli)
-	{
-		int	i;
-
-		for (i = 0; i < result->nalloc; i++)
-		{
-			zbx_free(result->values_cli[i]);
-		}
-
-		zbx_free(result->values);
-		zbx_free(result->values_cli);
-		zbx_free(result->values_len);
-	}
-
-	if (result->hstmt)
-		SQLFreeHandle(SQL_HANDLE_STMT, result->hstmt);
-
-	zbx_free(result);
-#elif defined(HAVE_MYSQL)
+#if defined(HAVE_MYSQL)
 	if (NULL == result)
 		return;
 
@@ -2239,54 +2135,6 @@ void	DBfree_result(DB_RESULT result)
 #endif	/* HAVE_SQLITE3 */
 }
 
-#ifdef HAVE_IBM_DB2
-/* server status: SQL_CD_TRUE or SQL_CD_FALSE */
-static int	IBM_DB2server_status()
-{
-	int	server_status = SQL_CD_TRUE;
-
-	if (SUCCEED != zbx_ibm_db2_success(SQLGetConnectAttr(ibm_db2.hdbc, SQL_ATTR_CONNECTION_DEAD, &server_status,
-			SQL_IS_POINTER, NULL)))
-	{
-		zabbix_log(LOG_LEVEL_WARNING, "cannot determine IBM DB2 server status, assuming not connected");
-	}
-
-	return (SQL_CD_FALSE == server_status ? SQL_CD_TRUE : SQL_CD_FALSE);
-}
-
-static int	zbx_ibm_db2_success(SQLRETURN ret)
-{
-	return (SQL_SUCCESS == ret || SQL_SUCCESS_WITH_INFO == ret ? SUCCEED : FAIL);
-}
-
-static int	zbx_ibm_db2_success_ext(SQLRETURN ret)
-{
-	return (SQL_SUCCESS == ret || SQL_SUCCESS_WITH_INFO == ret || SQL_NO_DATA_FOUND == ret ? SUCCEED : FAIL);
-}
-
-static void	zbx_ibm_db2_log_errors(SQLSMALLINT htype, SQLHANDLE hndl, zbx_err_codes_t err, const char *context)
-{
-	SQLCHAR		tmp_message[SQL_MAX_MESSAGE_LENGTH + 1], sqlstate[SQL_SQLSTATE_SIZE + 1];
-	char		*message = NULL;
-	SQLINTEGER	sqlcode = 0;
-	SQLSMALLINT	rec_nr = 1;
-	size_t		message_alloc = 0, message_offset = 0;
-
-	while (SQL_SUCCESS == SQLGetDiagRec(htype, hndl, rec_nr++, sqlstate, &sqlcode, tmp_message, sizeof(tmp_message),
-			NULL))
-	{
-		if (NULL != message)
-			zbx_chrcpy_alloc(&message, &message_alloc, &message_offset, '|');
-
-		zbx_snprintf_alloc(&message, &message_alloc, &message_offset, "[%s] %s", sqlstate, tmp_message);
-	}
-
-	zbx_db_errlog(err, (int)sqlcode, message, context);
-
-	zbx_free(message);
-}
-#endif
-
 #ifdef HAVE_ORACLE
 /* server status: OCI_SERVER_NORMAL or OCI_SERVER_NOT_CONNECTED */
 static ub4	OCI_DBserver_status(void)
@@ -2474,7 +2322,7 @@ static int	zbx_db_get_escape_like_pattern_len(const char *src)
  *           using '!' as escape character. Our queries then become:          *
  *                                                                            *
  *           ... LIKE 'a!_b!%c\\d\'e!!f' ESCAPE '!' (MySQL, PostgreSQL)       *
- *           ... LIKE 'a!_b!%c\d''e!!f' ESCAPE '!' (IBM DB2, Oracle, SQLite3) *
+ *           ... LIKE 'a!_b!%c\d''e!!f' ESCAPE '!' (Oracle, SQLite3)          *
  *                                                                            *
  *           Using backslash as escape character in LIKE would be too much    *
  *           trouble, because escaping backslashes would have to be escaped   *
@@ -2482,7 +2330,7 @@ static int	zbx_db_get_escape_like_pattern_len(const char *src)
  *                                                                            *
  *           ... LIKE 'a\\_b\\%c\\\\d\'e!f' ESCAPE '\\' or                    *
  *           ... LIKE 'a\\_b\\%c\\\\d\\\'e!f' ESCAPE '\\' (MySQL, PostgreSQL) *
- *           ... LIKE 'a\_b\%c\\d''e!f' ESCAPE '\' (IBM DB2, Oracle, SQLite3) *
+ *           ... LIKE 'a\_b\%c\\d''e!f' ESCAPE '\' (Oracle, SQLite3)          *
  *                                                                            *
  *           Hence '!' instead of backslash.                                  *
  *                                                                            *
@@ -2552,9 +2400,27 @@ char	*zbx_db_dyn_escape_like_pattern(const char *src)
  ******************************************************************************/
 int	zbx_db_strlen_n(const char *text, size_t maxlen)
 {
-#ifdef HAVE_IBM_DB2
-	return zbx_strlen_utf8_nbytes(text, maxlen);
-#else
 	return zbx_strlen_utf8_nchars(text, maxlen);
+}
+
+/******************************************************************************
+ *                                                                            *
+ * Function: zbx_dbms_get_version                                             *
+ *                                                                            *
+ * Purpose: returns DBMS version as integer: MMmmuu                           *
+ *          M = major version part                                            *
+ *          m = minor version part                                            *
+ *          u = micro version part                                            *
+ *                                                                            *
+ * Example: 1.2.34 version will be returned as 10234                          *
+ *                                                                            *
+ * Return value: DBMS version or 0 if unknown                                 *
+ *                                                                            *
+ ******************************************************************************/
+int	zbx_dbms_get_version(void)
+{
+#ifdef HAVE_POSTGRESQL
+	return ZBX_PG_SVERSION;
 #endif
+	return 0;
 }
