@@ -118,7 +118,7 @@ static history_value_t	history_str2value(char *str, unsigned char value_type)
 
 static const char	*history_value2str(const ZBX_DC_HISTORY *h)
 {
-	static char	buffer[MAX_ID_LEN + 1];
+	static char	buffer[ZBX_MAX_DOUBLE_LEN + 1];
 
 	switch (h->value_type)
 	{
@@ -128,7 +128,7 @@ static const char	*history_value2str(const ZBX_DC_HISTORY *h)
 		case ITEM_VALUE_TYPE_LOG:
 			return h->value.log->value;
 		case ITEM_VALUE_TYPE_FLOAT:
-			zbx_snprintf(buffer, sizeof(buffer), ZBX_FS_DBL, h->value.dbl);
+			zbx_snprintf(buffer, sizeof(buffer), ZBX_FS_DBL64, h->value.dbl);
 			break;
 		case ITEM_VALUE_TYPE_UINT64:
 			zbx_snprintf(buffer, sizeof(buffer), ZBX_FS_UI64, h->value.ui64);
@@ -262,15 +262,13 @@ static void	elastic_close(zbx_history_iface_t *hist)
  ******************************************************************************/
 static int	elastic_is_error_present(zbx_httppage_t *page, char **err)
 {
-	const char		*__function_name = "elastic_is_error_present";
-
 	struct zbx_json_parse	jp, jp_values, jp_index, jp_error, jp_items, jp_item;
 	const char		*errors, *p = NULL;
 	char			*index = NULL, *status = NULL, *type = NULL, *reason = NULL;
 	size_t			index_alloc = 0, status_alloc = 0, type_alloc = 0, reason_alloc = 0;
 	int			rc_js = SUCCEED;
 
-	zabbix_log(LOG_LEVEL_TRACE, "%s() raw json: %s", __function_name, ZBX_NULL2EMPTY_STR(page->data));
+	zabbix_log(LOG_LEVEL_TRACE, "%s() raw json: %s", __func__, ZBX_NULL2EMPTY_STR(page->data));
 
 	if (SUCCEED != zbx_json_open(page->data, &jp) || SUCCEED != zbx_json_brackets_open(jp.start, &jp_values))
 		return FAIL;
@@ -383,6 +381,8 @@ static void	elastic_writer_release(void)
 static void	elastic_writer_add_iface(zbx_history_iface_t *hist)
 {
 	zbx_elastic_data_t	*data = (zbx_elastic_data_t *)hist->data;
+	CURLoption		opt;
+	CURLcode		err;
 
 	elastic_writer_init();
 
@@ -391,22 +391,42 @@ static void	elastic_writer_add_iface(zbx_history_iface_t *hist)
 		zabbix_log(LOG_LEVEL_ERR, "cannot initialize cURL session");
 		return;
 	}
-	curl_easy_setopt(data->handle, CURLOPT_URL, data->post_url);
-	curl_easy_setopt(data->handle, CURLOPT_POST, 1L);
-	curl_easy_setopt(data->handle, CURLOPT_POSTFIELDS, data->buf);
-	curl_easy_setopt(data->handle, CURLOPT_WRITEFUNCTION, curl_write_cb);
-	curl_easy_setopt(data->handle, CURLOPT_WRITEDATA, &page_w[hist->value_type].page);
-	curl_easy_setopt(data->handle, CURLOPT_FAILONERROR, 1L);
-	curl_easy_setopt(data->handle, CURLOPT_ERRORBUFFER, page_w[hist->value_type].errbuf);
+
+	if (CURLE_OK != (err = curl_easy_setopt(data->handle, opt = CURLOPT_URL, data->post_url)) ||
+			CURLE_OK != (err = curl_easy_setopt(data->handle, opt = CURLOPT_POST, 1L)) ||
+			CURLE_OK != (err = curl_easy_setopt(data->handle, opt = CURLOPT_POSTFIELDS, data->buf)) ||
+			CURLE_OK != (err = curl_easy_setopt(data->handle, opt = CURLOPT_WRITEFUNCTION,
+					curl_write_cb)) ||
+			CURLE_OK != (err = curl_easy_setopt(data->handle, opt = CURLOPT_WRITEDATA,
+					&page_w[hist->value_type].page)) ||
+			CURLE_OK != (err = curl_easy_setopt(data->handle, opt = CURLOPT_FAILONERROR, 1L)) ||
+			CURLE_OK != (err = curl_easy_setopt(data->handle, opt = CURLOPT_ERRORBUFFER,
+					page_w[hist->value_type].errbuf)))
+	{
+		zabbix_log(LOG_LEVEL_ERR, "cannot set cURL option %d: [%s]", (int)opt, curl_easy_strerror(err));
+		goto out;
+	}
+
 	*page_w[hist->value_type].errbuf = '\0';
-	curl_easy_setopt(data->handle, CURLOPT_PRIVATE, &page_w[hist->value_type]);
+
+	if (CURLE_OK != (err = curl_easy_setopt(data->handle, opt = CURLOPT_PRIVATE, &page_w[hist->value_type])))
+	{
+		zabbix_log(LOG_LEVEL_ERR, "cannot set cURL option %d: [%s]", (int)opt, curl_easy_strerror(err));
+		goto out;
+	}
+
 	page_w[hist->value_type].page.offset = 0;
+
 	if (0 < page_w[hist->value_type].page.alloc)
 		*page_w[hist->value_type].page.data = '\0';
 
 	curl_multi_add_handle(writer.handle, data->handle);
 
 	zbx_vector_ptr_append(&writer.ifaces, hist);
+
+	return;
+out:
+	elastic_close(hist);
 }
 
 /************************************************************************************
@@ -418,19 +438,19 @@ static void	elastic_writer_add_iface(zbx_history_iface_t *hist)
  ************************************************************************************/
 static int	elastic_writer_flush(void)
 {
-	const char		*__function_name = "elastic_writer_flush";
-
 	struct curl_slist	*curl_headers = NULL;
 	int			i, running, previous, msgnum;
 	CURLMsg			*msg;
 	zbx_vector_ptr_t	retries;
+	CURLcode		err;
+	int			ret = SUCCEED;
 
-	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __function_name);
+	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __func__);
 
 	/* The writer might be uninitialized only if the history */
 	/* was already flushed. In that case, return SUCCEED */
 	if (0 == writer.initialized)
-		return SUCCEED;
+		goto end;
 
 	zbx_vector_ptr_create(&retries);
 
@@ -441,7 +461,13 @@ static int	elastic_writer_flush(void)
 		zbx_history_iface_t	*hist = (zbx_history_iface_t *)writer.ifaces.values[i];
 		zbx_elastic_data_t	*data = (zbx_elastic_data_t *)hist->data;
 
-		(void)curl_easy_setopt(data->handle, CURLOPT_HTTPHEADER, curl_headers);
+		if (CURLE_OK != (err = curl_easy_setopt(data->handle, CURLOPT_HTTPHEADER, curl_headers)))
+		{
+			zabbix_log(LOG_LEVEL_ERR, "cannot set cURL option %d: [%s]", (int)CURLOPT_HTTPHEADER,
+					curl_easy_strerror(err));
+			ret = FAIL;
+			goto clean;
+		}
 
 		zabbix_log(LOG_LEVEL_DEBUG, "sending %s", data->buf);
 	}
@@ -486,14 +512,13 @@ try_again:
 				else
 				{
 					char		http_status[MAX_STRING_LEN];
-					long int	err;
-					CURLcode	curl_err;
+					long int	response_code;
 
-					if (CURLE_OK == (curl_err = curl_easy_getinfo(msg->easy_handle,
-							CURLINFO_RESPONSE_CODE, &err)))
+					if (CURLE_OK == curl_easy_getinfo(msg->easy_handle,
+							CURLINFO_RESPONSE_CODE, &response_code))
 					{
 						zbx_snprintf(http_status, sizeof(http_status), "HTTP status code: %ld",
-								err);
+								response_code);
 					}
 					else
 					{
@@ -528,7 +553,7 @@ try_again:
 					&& SUCCEED == elastic_is_error_present(&curl_page->page, &error))
 			{
 				zabbix_log(LOG_LEVEL_WARNING, "%s() cannot send data to elasticsearch: %s",
-						__function_name, error);
+						__func__, error);
 				zbx_free(error);
 
 				/* If the error is due to elastic internal problems (for example an index */
@@ -556,16 +581,17 @@ try_again:
 		sleep(ZBX_HISTORY_STORAGE_DOWN / 1000);
 		goto try_again;
 	}
-
+clean:
 	curl_slist_free_all(curl_headers);
 
 	zbx_vector_ptr_destroy(&retries);
 
 	elastic_writer_release();
 
-	zabbix_log(LOG_LEVEL_DEBUG, "End of %s()", __function_name);
+end:
+	zabbix_log(LOG_LEVEL_DEBUG, "End of %s()", __func__);
 
-	return SUCCEED;
+	return ret;
 }
 
 /******************************************************************************************************************
@@ -616,8 +642,6 @@ static void	elastic_destroy(zbx_history_iface_t *hist)
 static int	elastic_get_values(zbx_history_iface_t *hist, zbx_uint64_t itemid, int start, int count, int end,
 		zbx_vector_history_record_t *values)
 {
-	const char		*__function_name = "elastic_get_values";
-
 	zbx_elastic_data_t	*data = (zbx_elastic_data_t *)hist->data;
 	size_t			url_alloc = 0, url_offset = 0, id_alloc = 0, scroll_alloc = 0, scroll_offset = 0;
 	int			total, empty, ret;
@@ -625,8 +649,9 @@ static int	elastic_get_values(zbx_history_iface_t *hist, zbx_uint64_t itemid, in
 	struct zbx_json		query;
 	struct curl_slist	*curl_headers = NULL;
 	char			*scroll_id = NULL, *scroll_query = NULL, errbuf[CURL_ERROR_SIZE];
+	CURLoption		opt;
 
-	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __function_name);
+	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __func__);
 
 	ret = FAIL;
 
@@ -637,7 +662,7 @@ static int	elastic_get_values(zbx_history_iface_t *hist, zbx_uint64_t itemid, in
 		return FAIL;
 	}
 
-	zbx_snprintf_alloc(&data->post_url, &url_alloc, &url_offset, "%s/%s*/values/_search?scroll=10s", data->base_url,
+	zbx_snprintf_alloc(&data->post_url, &url_alloc, &url_offset, "%s/%s*/_search?scroll=10s", data->base_url,
 			value_type_str[hist->value_type]);
 
 	/* prepare the json query for elasticsearch, apply ranges if needed */
@@ -685,13 +710,18 @@ static int	elastic_get_values(zbx_history_iface_t *hist, zbx_uint64_t itemid, in
 
 	curl_headers = curl_slist_append(curl_headers, "Content-Type: application/json");
 
-	curl_easy_setopt(data->handle, CURLOPT_URL, data->post_url);
-	curl_easy_setopt(data->handle, CURLOPT_POSTFIELDS, query.buffer);
-	curl_easy_setopt(data->handle, CURLOPT_WRITEFUNCTION, curl_write_cb);
-	curl_easy_setopt(data->handle, CURLOPT_WRITEDATA, &page_r);
-	curl_easy_setopt(data->handle, CURLOPT_HTTPHEADER, curl_headers);
-	curl_easy_setopt(data->handle, CURLOPT_FAILONERROR, 1L);
-	curl_easy_setopt(data->handle, CURLOPT_ERRORBUFFER, errbuf);
+	if (CURLE_OK != (err = curl_easy_setopt(data->handle, opt = CURLOPT_URL, data->post_url)) ||
+			CURLE_OK != (err = curl_easy_setopt(data->handle, opt = CURLOPT_POSTFIELDS, query.buffer)) ||
+			CURLE_OK != (err = curl_easy_setopt(data->handle, opt = CURLOPT_WRITEFUNCTION,
+					curl_write_cb)) ||
+			CURLE_OK != (err = curl_easy_setopt(data->handle, opt = CURLOPT_WRITEDATA, &page_r)) ||
+			CURLE_OK != (err = curl_easy_setopt(data->handle, opt = CURLOPT_HTTPHEADER, curl_headers)) ||
+			CURLE_OK != (err = curl_easy_setopt(data->handle, opt = CURLOPT_FAILONERROR, 1L)) ||
+			CURLE_OK != (err = curl_easy_setopt(data->handle, opt = CURLOPT_ERRORBUFFER, errbuf)))
+	{
+		zabbix_log(LOG_LEVEL_ERR, "cannot set cURL option %d: [%s]", (int)opt, curl_easy_strerror(err));
+		goto out;
+	}
 
 	zabbix_log(LOG_LEVEL_DEBUG, "sending query to %s; post data: %s", data->post_url, query.buffer);
 
@@ -706,7 +736,12 @@ static int	elastic_get_values(zbx_history_iface_t *hist, zbx_uint64_t itemid, in
 	url_offset = 0;
 	zbx_snprintf_alloc(&data->post_url, &url_alloc, &url_offset, "%s/_search/scroll", data->base_url);
 
-	curl_easy_setopt(data->handle, CURLOPT_URL, data->post_url);
+	if (CURLE_OK != (err = curl_easy_setopt(data->handle, CURLOPT_URL, data->post_url)))
+	{
+		zabbix_log(LOG_LEVEL_ERR, "cannot set cURL option %d: [%s]", (int)CURLOPT_URL,
+				curl_easy_strerror(err));
+		goto out;
+	}
 
 	total = (0 == count ? -1 : count);
 
@@ -772,7 +807,12 @@ static int	elastic_get_values(zbx_history_iface_t *hist, zbx_uint64_t itemid, in
 		zbx_snprintf_alloc(&scroll_query, &scroll_alloc, &scroll_offset,
 				"{\"scroll\":\"10s\",\"scroll_id\":\"%s\"}\n", ZBX_NULL2EMPTY_STR(scroll_id));
 
-		curl_easy_setopt(data->handle, CURLOPT_POSTFIELDS, scroll_query);
+		if (CURLE_OK != (err = curl_easy_setopt(data->handle, CURLOPT_POSTFIELDS, scroll_query)))
+		{
+			zabbix_log(LOG_LEVEL_ERR, "cannot set cURL option %d: [%s]", (int)CURLOPT_POSTFIELDS,
+					curl_easy_strerror(err));
+			break;
+		}
 
 		page_r.offset = 0;
 		*errbuf = '\0';
@@ -791,9 +831,16 @@ static int	elastic_get_values(zbx_history_iface_t *hist, zbx_uint64_t itemid, in
 		zbx_snprintf_alloc(&data->post_url, &url_alloc, &url_offset, "%s/_search/scroll/%s", data->base_url,
 				scroll_id);
 
-		curl_easy_setopt(data->handle, CURLOPT_URL, data->post_url);
-		curl_easy_setopt(data->handle, CURLOPT_POSTFIELDS, NULL);
-		curl_easy_setopt(data->handle, CURLOPT_CUSTOMREQUEST, "DELETE");
+		if (CURLE_OK != (err = curl_easy_setopt(data->handle, opt = CURLOPT_URL, data->post_url)) ||
+				CURLE_OK != (err = curl_easy_setopt(data->handle, opt = CURLOPT_POSTFIELDS, NULL)) ||
+				CURLE_OK != (err = curl_easy_setopt(data->handle, opt = CURLOPT_CUSTOMREQUEST, "DELETE")))
+		{
+			zabbix_log(LOG_LEVEL_ERR, "cannot set cURL option %d: [%s]", (int)opt,
+					curl_easy_strerror(err));
+			ret = FAIL;
+			goto out;
+		}
+
 
 		zabbix_log(LOG_LEVEL_DEBUG, "elasticsearch closing scroll %s", data->post_url);
 
@@ -815,7 +862,7 @@ out:
 
 	zbx_vector_history_record_sort(values, (zbx_compare_func_t)zbx_history_record_compare_desc_func);
 
-	zabbix_log(LOG_LEVEL_DEBUG, "End of %s()", __function_name);
+	zabbix_log(LOG_LEVEL_DEBUG, "End of %s()", __func__);
 
 	return ret;
 }
@@ -832,8 +879,6 @@ out:
  ************************************************************************************/
 static int	elastic_add_values(zbx_history_iface_t *hist, const zbx_vector_ptr_t *history)
 {
-	const char	*__function_name = "elastic_add_values";
-
 	zbx_elastic_data_t	*data = (zbx_elastic_data_t *)hist->data;
 	int			i, num = 0;
 	ZBX_DC_HISTORY		*h;
@@ -841,13 +886,12 @@ static int	elastic_add_values(zbx_history_iface_t *hist, const zbx_vector_ptr_t
 	size_t			buf_alloc = 0, buf_offset = 0;
 	char			pipeline[14]; /* index name length + suffix "-pipeline" */
 
-	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __function_name);
+	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __func__);
 
 	zbx_json_init(&json_idx, ZBX_IDX_JSON_ALLOCATE);
 
 	zbx_json_addobject(&json_idx, "index");
 	zbx_json_addstring(&json_idx, "_index", value_type_str[hist->value_type], ZBX_JSON_TYPE_STRING);
-	zbx_json_addstring(&json_idx, "_type", "values", ZBX_JSON_TYPE_STRING);
 
 	if (1 == CONFIG_HISTORY_STORAGE_PIPELINES)
 	{
@@ -904,7 +948,7 @@ static int	elastic_add_values(zbx_history_iface_t *hist, const zbx_vector_ptr_t
 
 	zbx_json_free(&json_idx);
 
-	zabbix_log(LOG_LEVEL_DEBUG, "End of %s()", __function_name);
+	zabbix_log(LOG_LEVEL_DEBUG, "End of %s()", __func__);
 
 	return num;
 }
