@@ -25,6 +25,7 @@
 #include "actions.h"
 #include "operations.h"
 #include "events.h"
+#include "zbxregexp.h"
 
 /******************************************************************************
  *                                                                            *
@@ -110,14 +111,13 @@ static int	check_condition_event_tag_value(const DB_EVENT *event, DB_CONDITION *
  ******************************************************************************/
 static int	check_trigger_condition(const DB_EVENT *event, DB_CONDITION *condition)
 {
-	const char	*__function_name = "check_trigger_condition";
 	DB_RESULT	result;
 	DB_ROW		row;
 	zbx_uint64_t	condition_value;
 	char		*tmp_str = NULL;
 	int		ret = FAIL;
 
-	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __function_name);
+	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __func__);
 
 	if (CONDITION_TYPE_HOST_GROUP == condition->conditiontype)
 	{
@@ -188,7 +188,7 @@ static int	check_trigger_condition(const DB_EVENT *event, DB_CONDITION *conditio
 
 					zabbix_log(LOG_LEVEL_DEBUG, "%s() check host template condition,"
 							" selecting parent triggerid:" ZBX_FS_UI64,
-							__function_name, triggerid);
+							__func__, triggerid);
 				}
 				DBfree_result(result);
 
@@ -488,7 +488,7 @@ static int	check_trigger_condition(const DB_EVENT *event, DB_CONDITION *conditio
 		ret = FAIL;
 	}
 
-	zabbix_log(LOG_LEVEL_DEBUG, "End of %s():%s", __function_name, zbx_result_string(ret));
+	zabbix_log(LOG_LEVEL_DEBUG, "End of %s():%s", __func__, zbx_result_string(ret));
 
 	return ret;
 }
@@ -510,13 +510,12 @@ static int	check_trigger_condition(const DB_EVENT *event, DB_CONDITION *conditio
  ******************************************************************************/
 static int	check_discovery_condition(const DB_EVENT *event, DB_CONDITION *condition)
 {
-	const char	*__function_name = "check_discovery_condition";
 	DB_RESULT	result;
 	DB_ROW		row;
 	zbx_uint64_t	condition_value;
 	int		tmp_int, ret = FAIL;
 
-	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __function_name);
+	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __func__);
 
 	if (CONDITION_TYPE_DRULE == condition->conditiontype)
 	{
@@ -866,19 +865,19 @@ static int	check_discovery_condition(const DB_EVENT *event, DB_CONDITION *condit
 		ret = FAIL;
 	}
 
-	zabbix_log(LOG_LEVEL_DEBUG, "End of %s():%s", __function_name, zbx_result_string(ret));
+	zabbix_log(LOG_LEVEL_DEBUG, "End of %s():%s", __func__, zbx_result_string(ret));
 
 	return ret;
 }
 
 /******************************************************************************
  *                                                                            *
- * Function: check_auto_registration_condition                                *
+ * Function: check_autoregistration_condition                                 *
  *                                                                            *
  * Purpose: check if event matches single condition                           *
  *                                                                            *
- * Parameters: event - auto registration event to check                       *
- *                         (event->source == EVENT_SOURCE_AUTO_REGISTRATION)  *
+ * Parameters: event - autoregistration event to check                        *
+ *                         (event->source == EVENT_SOURCE_AUTOREGISTRATION)   *
  *             condition - condition for matching                             *
  *                                                                            *
  * Return value: SUCCEED - matches, FAIL - otherwise                          *
@@ -886,16 +885,15 @@ static int	check_discovery_condition(const DB_EVENT *event, DB_CONDITION *condit
  * Author: Alexei Vladishev                                                   *
  *                                                                            *
  ******************************************************************************/
-static int	check_auto_registration_condition(const DB_EVENT *event, DB_CONDITION *condition)
+static int	check_autoregistration_condition(const DB_EVENT *event, DB_CONDITION *condition)
 {
-	const char	*__function_name = "check_auto_registration_condition";
 	DB_RESULT	result;
 	DB_ROW		row;
 	zbx_uint64_t	condition_value, id;
 	int		ret = FAIL;
 	const char	*condition_field;
 
-	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __function_name);
+	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __func__);
 
 	switch (condition->conditiontype)
 	{
@@ -924,6 +922,14 @@ static int	check_auto_registration_condition(const DB_EVENT *event, DB_CONDITION
 						if (NULL == strstr(row[0], condition->value))
 							ret = SUCCEED;
 						break;
+					case CONDITION_OPERATOR_REGEXP:
+						if (NULL != zbx_regexp_match(row[0], condition->value, NULL))
+							ret = SUCCEED;
+						break;
+					case CONDITION_OPERATOR_NOT_REGEXP:
+						if (NULL == zbx_regexp_match(row[0], condition->value, NULL))
+							ret = SUCCEED;
+						break;
 					default:
 						ret = NOTSUPPORTED;
 				}
@@ -973,7 +979,7 @@ static int	check_auto_registration_condition(const DB_EVENT *event, DB_CONDITION
 		ret = FAIL;
 	}
 
-	zabbix_log(LOG_LEVEL_DEBUG, "End of %s():%s", __function_name, zbx_result_string(ret));
+	zabbix_log(LOG_LEVEL_DEBUG, "End of %s():%s", __func__, zbx_result_string(ret));
 
 	return ret;
 }
@@ -992,14 +998,13 @@ static int	check_auto_registration_condition(const DB_EVENT *event, DB_CONDITION
  ******************************************************************************/
 static int	check_internal_condition(const DB_EVENT *event, DB_CONDITION *condition)
 {
-	const char	*__function_name = "check_internal_condition";
 	DB_RESULT	result;
 	DB_ROW		row;
 	zbx_uint64_t	condition_value;
 	int		ret = FAIL;
 	char		sql[256];
 
-	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __function_name);
+	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __func__);
 
 	if (EVENT_OBJECT_TRIGGER != event->object && EVENT_OBJECT_ITEM != event->object &&
 			EVENT_OBJECT_LLDRULE != event->object)
@@ -1127,8 +1132,7 @@ static int	check_internal_condition(const DB_EVENT *event, DB_CONDITION *conditi
 					ZBX_STR2UINT64(objectid, row[0]);
 
 					zabbix_log(LOG_LEVEL_DEBUG, "%s() check host template condition,"
-							" selecting parent objectid:" ZBX_FS_UI64,
-							__function_name, objectid);
+							" selecting parent objectid:" ZBX_FS_UI64, __func__, objectid);
 				}
 				DBfree_result(result);
 
@@ -1294,7 +1298,7 @@ static int	check_internal_condition(const DB_EVENT *event, DB_CONDITION *conditi
 		ret = FAIL;
 	}
 out:
-	zabbix_log(LOG_LEVEL_DEBUG, "End of %s():%s", __function_name, zbx_result_string(ret));
+	zabbix_log(LOG_LEVEL_DEBUG, "End of %s():%s", __func__, zbx_result_string(ret));
 
 	return ret;
 }
@@ -1315,11 +1319,10 @@ out:
  ******************************************************************************/
 int	check_action_condition(const DB_EVENT *event, DB_CONDITION *condition)
 {
-	const char	*__function_name = "check_action_condition";
-	int		ret = FAIL;
+	int	ret = FAIL;
 
 	zabbix_log(LOG_LEVEL_DEBUG, "In %s() actionid:" ZBX_FS_UI64 " conditionid:" ZBX_FS_UI64 " cond.value:'%s'"
-			" cond.value2:'%s'", __function_name, condition->actionid, condition->conditionid,
+			" cond.value2:'%s'", __func__, condition->actionid, condition->conditionid,
 			ZBX_NULL2STR(condition->value), ZBX_NULL2STR(condition->value2));
 
 	switch (event->source)
@@ -1330,8 +1333,8 @@ int	check_action_condition(const DB_EVENT *event, DB_CONDITION *condition)
 		case EVENT_SOURCE_DISCOVERY:
 			ret = check_discovery_condition(event, condition);
 			break;
-		case EVENT_SOURCE_AUTO_REGISTRATION:
-			ret = check_auto_registration_condition(event, condition);
+		case EVENT_SOURCE_AUTOREGISTRATION:
+			ret = check_autoregistration_condition(event, condition);
 			break;
 		case EVENT_SOURCE_INTERNAL:
 			ret = check_internal_condition(event, condition);
@@ -1341,7 +1344,7 @@ int	check_action_condition(const DB_EVENT *event, DB_CONDITION *condition)
 					event->source, condition->conditionid);
 	}
 
-	zabbix_log(LOG_LEVEL_DEBUG, "End of %s():%s", __function_name, zbx_result_string(ret));
+	zabbix_log(LOG_LEVEL_DEBUG, "End of %s():%s", __func__, zbx_result_string(ret));
 
 	return ret;
 }
@@ -1362,15 +1365,13 @@ int	check_action_condition(const DB_EVENT *event, DB_CONDITION *condition)
  ******************************************************************************/
 static int	check_action_conditions(zbx_action_eval_t *action)
 {
-	const char	*__function_name = "check_action_conditions";
-
 	DB_CONDITION	*condition;
 	int		condition_result, ret = SUCCEED, id_len, i;
 	unsigned char	old_type = 0xff;
 	char		*expression = NULL, tmp[ZBX_MAX_UINT64_LEN + 2], *ptr, error[256];
 	double		eval_result;
 
-	zabbix_log(LOG_LEVEL_DEBUG, "In %s() actionid:" ZBX_FS_UI64, __function_name, action->actionid);
+	zabbix_log(LOG_LEVEL_DEBUG, "In %s() actionid:" ZBX_FS_UI64, __func__, action->actionid);
 
 	if (CONDITION_EVAL_TYPE_EXPRESSION == action->evaltype)
 		expression = zbx_strdup(expression, action->formula);
@@ -1448,7 +1449,7 @@ static int	check_action_conditions(zbx_action_eval_t *action)
 	}
 clean:
 
-	zabbix_log(LOG_LEVEL_DEBUG, "End of %s():%s", __function_name, zbx_result_string(ret));
+	zabbix_log(LOG_LEVEL_DEBUG, "End of %s():%s", __func__, zbx_result_string(ret));
 
 	return ret;
 }
@@ -1470,15 +1471,14 @@ clean:
  ******************************************************************************/
 static void	execute_operations(const DB_EVENT *event, zbx_uint64_t actionid)
 {
-	const char		*__function_name = "execute_operations";
-
 	DB_RESULT		result;
 	DB_ROW			row;
 	zbx_uint64_t		groupid, templateid;
 	zbx_vector_uint64_t	lnk_templateids, del_templateids,
 				new_groupids, del_groupids;
+	int			i;
 
-	zabbix_log(LOG_LEVEL_DEBUG, "In %s() actionid:" ZBX_FS_UI64, __function_name, actionid);
+	zabbix_log(LOG_LEVEL_DEBUG, "In %s() actionid:" ZBX_FS_UI64, __func__, actionid);
 
 	zbx_vector_uint64_create(&lnk_templateids);
 	zbx_vector_uint64_create(&del_templateids);
@@ -1491,7 +1491,8 @@ static void	execute_operations(const DB_EVENT *event, zbx_uint64_t actionid)
 				" left join opgroup g on g.operationid=o.operationid"
 				" left join optemplate t on t.operationid=o.operationid"
 				" left join opinventory oi on oi.operationid=o.operationid"
-			" where o.actionid=" ZBX_FS_UI64,
+			" where o.actionid=" ZBX_FS_UI64
+			" order by o.operationid",
 			actionid);
 
 	while (NULL != (row = DBfetch(result)))
@@ -1528,11 +1529,27 @@ static void	execute_operations(const DB_EVENT *event, zbx_uint64_t actionid)
 				break;
 			case OPERATION_TYPE_TEMPLATE_ADD:
 				if (0 != templateid)
+				{
+					if (FAIL != (i = zbx_vector_uint64_search(&del_templateids, templateid,
+							ZBX_DEFAULT_UINT64_COMPARE_FUNC)))
+					{
+						zbx_vector_uint64_remove(&del_templateids, i);
+					}
+
 					zbx_vector_uint64_append(&lnk_templateids, templateid);
+				}
 				break;
 			case OPERATION_TYPE_TEMPLATE_REMOVE:
 				if (0 != templateid)
+				{
+					if (FAIL != (i = zbx_vector_uint64_search(&lnk_templateids, templateid,
+							ZBX_DEFAULT_UINT64_COMPARE_FUNC)))
+					{
+						zbx_vector_uint64_remove(&lnk_templateids, i);
+					}
+
 					zbx_vector_uint64_append(&del_templateids, templateid);
+				}
 				break;
 			case OPERATION_TYPE_HOST_INVENTORY:
 				op_host_inventory_mode(event, inventory_mode);
@@ -1543,13 +1560,6 @@ static void	execute_operations(const DB_EVENT *event, zbx_uint64_t actionid)
 	}
 	DBfree_result(result);
 
-	if (0 != lnk_templateids.values_num)
-	{
-		zbx_vector_uint64_sort(&lnk_templateids, ZBX_DEFAULT_UINT64_COMPARE_FUNC);
-		zbx_vector_uint64_uniq(&lnk_templateids, ZBX_DEFAULT_UINT64_COMPARE_FUNC);
-		op_template_add(event, &lnk_templateids);
-	}
-
 	if (0 != del_templateids.values_num)
 	{
 		zbx_vector_uint64_sort(&del_templateids, ZBX_DEFAULT_UINT64_COMPARE_FUNC);
@@ -1557,6 +1567,13 @@ static void	execute_operations(const DB_EVENT *event, zbx_uint64_t actionid)
 		op_template_del(event, &del_templateids);
 	}
 
+	if (0 != lnk_templateids.values_num)
+	{
+		zbx_vector_uint64_sort(&lnk_templateids, ZBX_DEFAULT_UINT64_COMPARE_FUNC);
+		zbx_vector_uint64_uniq(&lnk_templateids, ZBX_DEFAULT_UINT64_COMPARE_FUNC);
+		op_template_add(event, &lnk_templateids);
+	}
+
 	if (0 != new_groupids.values_num)
 	{
 		zbx_vector_uint64_sort(&new_groupids, ZBX_DEFAULT_UINT64_COMPARE_FUNC);
@@ -1576,7 +1593,7 @@ static void	execute_operations(const DB_EVENT *event, zbx_uint64_t actionid)
 	zbx_vector_uint64_destroy(&del_templateids);
 	zbx_vector_uint64_destroy(&lnk_templateids);
 
-	zabbix_log(LOG_LEVEL_DEBUG, "End of %s()", __function_name);
+	zabbix_log(LOG_LEVEL_DEBUG, "End of %s()", __func__);
 }
 
 /* data structures used to create new and recover existing escalations */
@@ -1726,16 +1743,13 @@ static int	check_event_conditions(const DB_EVENT *event, zbx_hashset_t *uniq_con
  ******************************************************************************/
 void	process_actions(const zbx_vector_ptr_t *events, const zbx_vector_uint64_pair_t *closed_events)
 {
-	const char			*__function_name = "process_actions";
-
 	int				i;
 	zbx_vector_ptr_t		actions;
 	zbx_vector_ptr_t 		new_escalations;
 	zbx_vector_uint64_pair_t	rec_escalations;
 	zbx_hashset_t			uniq_conditions[EVENT_SOURCE_COUNT];
 
-	zabbix_log(LOG_LEVEL_DEBUG, "In %s() events_num:" ZBX_FS_SIZE_T, __function_name,
-			(zbx_fs_size_t)events->values_num);
+	zabbix_log(LOG_LEVEL_DEBUG, "In %s() events_num:" ZBX_FS_SIZE_T, __func__, (zbx_fs_size_t)events->values_num);
 
 	zbx_vector_ptr_create(&new_escalations);
 	zbx_vector_uint64_pair_create(&rec_escalations);
@@ -1747,7 +1761,7 @@ void	process_actions(const zbx_vector_ptr_t *events, const zbx_vector_uint64_pai
 	zbx_dc_get_actions_eval(&actions, uniq_conditions, ZBX_ACTION_OPCLASS_NORMAL | ZBX_ACTION_OPCLASS_RECOVERY);
 
 	/* 1. All event sources: match PROBLEM events to action conditions, add them to 'new_escalations' list.      */
-	/* 2. EVENT_SOURCE_DISCOVERY, EVENT_SOURCE_AUTO_REGISTRATION: execute operations (except command and message */
+	/* 2. EVENT_SOURCE_DISCOVERY, EVENT_SOURCE_AUTOREGISTRATION: execute operations (except command and message  */
 	/*    operations) for events that match action conditions.                                                   */
 	for (i = 0; i < events->values_num; i++)
 	{
@@ -1780,14 +1794,14 @@ void	process_actions(const zbx_vector_ptr_t *events, const zbx_vector_uint64_pai
 					zbx_escalation_new_t	*new_escalation;
 
 					/* command and message operations handled by escalators even for    */
-					/* EVENT_SOURCE_DISCOVERY and EVENT_SOURCE_AUTO_REGISTRATION events */
+					/* EVENT_SOURCE_DISCOVERY and EVENT_SOURCE_AUTOREGISTRATION events  */
 					new_escalation = (zbx_escalation_new_t *)zbx_malloc(NULL, sizeof(zbx_escalation_new_t));
 					new_escalation->actionid = action->actionid;
 					new_escalation->event = event;
 					zbx_vector_ptr_append(&new_escalations, new_escalation);
 
 					if (EVENT_SOURCE_DISCOVERY == event->source ||
-							EVENT_SOURCE_AUTO_REGISTRATION == event->source)
+							EVENT_SOURCE_AUTOREGISTRATION == event->source)
 					{
 						execute_operations(event, action->actionid);
 					}
@@ -1928,7 +1942,7 @@ void	process_actions(const zbx_vector_ptr_t *events, const zbx_vector_uint64_pai
 	zbx_vector_uint64_pair_destroy(&rec_escalations);
 	zbx_vector_ptr_destroy(&new_escalations);
 
-	zabbix_log(LOG_LEVEL_DEBUG, "End of %s()", __function_name);
+	zabbix_log(LOG_LEVEL_DEBUG, "End of %s()", __func__);
 }
 
 /******************************************************************************
@@ -1942,8 +1956,6 @@ void	process_actions(const zbx_vector_ptr_t *events, const zbx_vector_uint64_pai
  ******************************************************************************/
 int	process_actions_by_acknowledgements(const zbx_vector_ptr_t *ack_tasks)
 {
-	const char		*__function_name = "process_actions_by_acknowledgements";
-
 	zbx_vector_ptr_t	actions;
 	zbx_hashset_t		uniq_conditions[EVENT_SOURCE_COUNT];
 	int			i, j, k, processed_num = 0, knext = 0;
@@ -1952,7 +1964,7 @@ int	process_actions_by_acknowledgements(const zbx_vector_ptr_t *ack_tasks)
 	zbx_vector_ptr_t	ack_escalations, events;
 	zbx_ack_escalation_t	*ack_escalation;
 
-	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __function_name);
+	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __func__);
 
 	zbx_vector_ptr_create(&ack_escalations);
 
@@ -2066,7 +2078,7 @@ out:
 	zbx_vector_ptr_clear_ext(&ack_escalations, zbx_ptr_free);
 	zbx_vector_ptr_destroy(&ack_escalations);
 
-	zabbix_log(LOG_LEVEL_DEBUG, "End of %s() processed_num:%d", __function_name, processed_num);
+	zabbix_log(LOG_LEVEL_DEBUG, "End of %s() processed_num:%d", __func__, processed_num);
 
 	return processed_num;
 }
@@ -2097,8 +2109,7 @@ void	get_db_actions_info(zbx_vector_uint64_t *actionids, zbx_vector_ptr_t *actio
 	DBadd_condition_alloc(&filter, &filter_alloc, &filter_offset, "actionid", actionids->values,
 			actionids->values_num);
 
-	result = DBselect("select actionid,name,status,eventsource,esc_period,def_shortdata,def_longdata,r_shortdata,"
-				"r_longdata,pause_suppressed,ack_shortdata,ack_longdata"
+	result = DBselect("select actionid,name,status,eventsource,esc_period,pause_suppressed"
 				" from actions"
 				" where%s order by actionid", filter);
 
@@ -2122,13 +2133,7 @@ void	get_db_actions_info(zbx_vector_uint64_t *actionids, zbx_vector_ptr_t *actio
 		}
 		zbx_free(tmp);
 
-		action->shortdata = zbx_strdup(NULL, row[5]);
-		action->longdata = zbx_strdup(NULL, row[6]);
-		action->r_shortdata = zbx_strdup(NULL, row[7]);
-		action->r_longdata = zbx_strdup(NULL, row[8]);
-		ZBX_STR2UCHAR(action->pause_suppressed, row[9]);
-		action->ack_shortdata = zbx_strdup(NULL, row[10]);
-		action->ack_longdata = zbx_strdup(NULL, row[11]);
+		ZBX_STR2UCHAR(action->pause_suppressed, row[5]);
 		action->name = zbx_strdup(NULL, row[1]);
 		action->recovery = ZBX_ACTION_RECOVERY_NONE;
 
@@ -2158,13 +2163,6 @@ void	get_db_actions_info(zbx_vector_uint64_t *actionids, zbx_vector_ptr_t *actio
 
 void	free_db_action(DB_ACTION *action)
 {
-	zbx_free(action->shortdata);
-	zbx_free(action->longdata);
-	zbx_free(action->r_shortdata);
-	zbx_free(action->r_longdata);
-	zbx_free(action->ack_shortdata);
-	zbx_free(action->ack_longdata);
 	zbx_free(action->name);
-
 	zbx_free(action);
 }
