@@ -41,13 +41,17 @@
 #include "checks_java.h"
 #include "checks_calculated.h"
 #include "checks_http.h"
-#include "../../libs/zbxcrypto/tls.h"
+#include "zbxcrypto.h"
 #include "zbxjson.h"
 #include "zbxhttp.h"
 
 extern unsigned char	process_type, program_type;
 extern int		server_num, process_num;
 
+#ifdef HAVE_NETSNMP
+static volatile sig_atomic_t	snmp_cache_reload_requested;
+#endif
+
 /******************************************************************************
  *                                                                            *
  * Function: db_host_update_availability                                      *
@@ -210,9 +214,7 @@ static unsigned char	host_availability_agent_by_item_type(unsigned char type)
 		case ITEM_TYPE_ZABBIX:
 			return ZBX_AGENT_ZABBIX;
 			break;
-		case ITEM_TYPE_SNMPv1:
-		case ITEM_TYPE_SNMPv2c:
-		case ITEM_TYPE_SNMPv3:
+		case ITEM_TYPE_SNMP:
 			return ZBX_AGENT_SNMP;
 			break;
 		case ITEM_TYPE_IPMI:
@@ -228,12 +230,11 @@ static unsigned char	host_availability_agent_by_item_type(unsigned char type)
 
 void	zbx_activate_item_host(DC_ITEM *item, zbx_timespec_t *ts)
 {
-	const char		*__function_name = "zbx_activate_item_host";
 	zbx_host_availability_t	in, out;
 	unsigned char		agent_type;
 
 	zabbix_log(LOG_LEVEL_DEBUG, "In %s() hostid:" ZBX_FS_UI64 " itemid:" ZBX_FS_UI64 " type:%d",
-			__function_name, item->host.hostid, item->itemid, (int)item->type);
+			__func__, item->host.hostid, item->itemid, (int)item->type);
 
 	zbx_host_availability_init(&in, item->host.hostid);
 	zbx_host_availability_init(&out, item->host.hostid);
@@ -266,17 +267,16 @@ out:
 	zbx_host_availability_clean(&out);
 	zbx_host_availability_clean(&in);
 
-	zabbix_log(LOG_LEVEL_DEBUG, "End of %s()", __function_name);
+	zabbix_log(LOG_LEVEL_DEBUG, "End of %s()", __func__);
 }
 
 void	zbx_deactivate_item_host(DC_ITEM *item, zbx_timespec_t *ts, const char *error)
 {
-	const char		*__function_name = "zbx_deactivate_item_host";
 	zbx_host_availability_t	in, out;
 	unsigned char		agent_type;
 
 	zabbix_log(LOG_LEVEL_DEBUG, "In %s() hostid:" ZBX_FS_UI64 " itemid:" ZBX_FS_UI64 " type:%d",
-			__function_name, item->host.hostid, item->itemid, (int)item->type);
+			__func__, item->host.hostid, item->itemid, (int)item->type);
 
 	zbx_host_availability_init(&in, item->host.hostid);
 	zbx_host_availability_init(&out,item->host.hostid);
@@ -325,16 +325,16 @@ void	zbx_deactivate_item_host(DC_ITEM *item, zbx_timespec_t *ts, const char *err
 		}
 	}
 
-	zabbix_log(LOG_LEVEL_DEBUG, "%s() errors_from:%d available:%d", __function_name,
+	zabbix_log(LOG_LEVEL_DEBUG, "%s() errors_from:%d available:%d", __func__,
 			out.agents[agent_type].errors_from, out.agents[agent_type].available);
 out:
 	zbx_host_availability_clean(&out);
 	zbx_host_availability_clean(&in);
 
-	zabbix_log(LOG_LEVEL_DEBUG, "End of %s()", __function_name);
+	zabbix_log(LOG_LEVEL_DEBUG, "End of %s()", __func__);
 }
 
-static void	free_result_ptr(AGENT_RESULT *result)
+void	zbx_free_result_ptr(AGENT_RESULT *result)
 {
 	free_result(result);
 	zbx_free(result);
@@ -342,10 +342,9 @@ static void	free_result_ptr(AGENT_RESULT *result)
 
 static int	get_value(DC_ITEM *item, AGENT_RESULT *result, zbx_vector_ptr_t *add_results)
 {
-	const char	*__function_name = "get_value";
-	int		res = FAIL;
+	int	res = FAIL;
 
-	zabbix_log(LOG_LEVEL_DEBUG, "In %s() key:'%s'", __function_name, item->key_orig);
+	zabbix_log(LOG_LEVEL_DEBUG, "In %s() key:'%s'", __func__, item->key_orig);
 
 	switch (item->type)
 	{
@@ -416,12 +415,12 @@ static int	get_value(DC_ITEM *item, AGENT_RESULT *result, zbx_vector_ptr_t *add_
 		zabbix_log(LOG_LEVEL_DEBUG, "Item [%s:%s] error: %s", item->host.host, item->key_orig, result->msg);
 	}
 
-	zabbix_log(LOG_LEVEL_DEBUG, "End of %s():%s", __function_name, zbx_result_string(res));
+	zabbix_log(LOG_LEVEL_DEBUG, "End of %s():%s", __func__, zbx_result_string(res));
 
 	return res;
 }
 
-static int	parse_query_fields(const DC_ITEM *item, char **query_fields)
+static int	parse_query_fields(const DC_ITEM *item, char **query_fields, unsigned char expand_macros)
 {
 	struct zbx_json_parse	jp_array, jp_object;
 	char			name[MAX_STRING_LEN], value[MAX_STRING_LEN];
@@ -464,15 +463,22 @@ static int	parse_query_fields(const DC_ITEM *item, char **query_fields)
 			zbx_chrcpy_alloc(query_fields, &alloc_len, &offset, '&');
 
 		data = zbx_strdup(data, name);
-		substitute_simple_macros(NULL, NULL, NULL,NULL, NULL, &item->host, item, NULL, NULL, &data,
-				MACRO_TYPE_HTTP_RAW, NULL, 0);
+		if (MACRO_EXPAND_YES == expand_macros)
+		{
+			substitute_simple_macros(NULL, NULL, NULL,NULL, NULL, &item->host, item, NULL, NULL, &data,
+					MACRO_TYPE_HTTP_RAW, NULL, 0);
+		}
 		zbx_http_url_encode(data, &data);
 		zbx_strcpy_alloc(query_fields, &alloc_len, &offset, data);
 		zbx_chrcpy_alloc(query_fields, &alloc_len, &offset, '=');
 
 		data = zbx_strdup(data, value);
-		substitute_simple_macros(NULL, NULL, NULL,NULL, NULL, &item->host, item, NULL, NULL, &data,
-				MACRO_TYPE_HTTP_RAW, NULL, 0);
+		if (MACRO_EXPAND_YES == expand_macros)
+		{
+			substitute_simple_macros_unmasked(NULL, NULL, NULL,NULL, NULL, &item->host, item, NULL, NULL,
+					&data, MACRO_TYPE_HTTP_RAW, NULL, 0);
+		}
+
 		zbx_http_url_encode(data, &data);
 		zbx_strcpy_alloc(query_fields, &alloc_len, &offset, data);
 
@@ -483,6 +489,314 @@ static int	parse_query_fields(const DC_ITEM *item, char **query_fields)
 	return SUCCEED;
 }
 
+void	zbx_prepare_items(DC_ITEM *items, int *errcodes, int num, AGENT_RESULT *results, unsigned char expand_macros)
+{
+	int	i;
+	char	*port = NULL, error[ITEM_ERROR_LEN_MAX];
+
+	for (i = 0; i < num; i++)
+	{
+		init_result(&results[i]);
+		errcodes[i] = SUCCEED;
+
+		ZBX_STRDUP(items[i].key, items[i].key_orig);
+		if (MACRO_EXPAND_YES == expand_macros && SUCCEED != substitute_key_macros_unmasked(&items[i].key, NULL,
+				&items[i], NULL, NULL, MACRO_TYPE_ITEM_KEY, error, sizeof(error)))
+		{
+			SET_MSG_RESULT(&results[i], zbx_strdup(NULL, error));
+			errcodes[i] = CONFIG_ERROR;
+			continue;
+		}
+
+		switch (items[i].type)
+		{
+			case ITEM_TYPE_ZABBIX:
+			case ITEM_TYPE_SNMP:
+			case ITEM_TYPE_JMX:
+				ZBX_STRDUP(port, items[i].interface.port_orig);
+				if (MACRO_EXPAND_YES == expand_macros)
+				{
+					substitute_simple_macros(NULL, NULL, NULL, NULL, &items[i].host.hostid, NULL,
+							NULL, NULL, NULL, &port, MACRO_TYPE_COMMON, NULL, 0);
+				}
+
+				if (FAIL == is_ushort(port, &items[i].interface.port))
+				{
+					SET_MSG_RESULT(&results[i], zbx_dsprintf(NULL, "Invalid port number [%s]",
+								items[i].interface.port_orig));
+					errcodes[i] = CONFIG_ERROR;
+					continue;
+				}
+				break;
+		}
+
+		switch (items[i].type)
+		{
+			case ITEM_TYPE_SNMP:
+				if (ZBX_IF_SNMP_VERSION_3 == items[i].snmp_version)
+				{
+					ZBX_STRDUP(items[i].snmpv3_securityname, items[i].snmpv3_securityname_orig);
+					ZBX_STRDUP(items[i].snmpv3_authpassphrase, items[i].snmpv3_authpassphrase_orig);
+					ZBX_STRDUP(items[i].snmpv3_privpassphrase, items[i].snmpv3_privpassphrase_orig);
+					ZBX_STRDUP(items[i].snmpv3_contextname, items[i].snmpv3_contextname_orig);
+
+					if (MACRO_EXPAND_YES == expand_macros)
+					{
+						substitute_simple_macros_unmasked(NULL, NULL, NULL, NULL, &items[i].host.hostid,
+								NULL, NULL, NULL, NULL, &items[i].snmpv3_securityname,
+								MACRO_TYPE_COMMON, NULL, 0);
+						substitute_simple_macros_unmasked(NULL, NULL, NULL, NULL, &items[i].host.hostid,
+								NULL, NULL, NULL, NULL, &items[i].snmpv3_authpassphrase,
+								MACRO_TYPE_COMMON, NULL, 0);
+						substitute_simple_macros_unmasked(NULL, NULL, NULL, NULL, &items[i].host.hostid,
+								NULL, NULL, NULL, NULL, &items[i].snmpv3_privpassphrase,
+								MACRO_TYPE_COMMON, NULL, 0);
+						substitute_simple_macros_unmasked(NULL, NULL, NULL, NULL, &items[i].host.hostid,
+								NULL, NULL, NULL, NULL, &items[i].snmpv3_contextname,
+								MACRO_TYPE_COMMON, NULL, 0);
+					}
+				}
+
+				ZBX_STRDUP(items[i].snmp_community, items[i].snmp_community_orig);
+				ZBX_STRDUP(items[i].snmp_oid, items[i].snmp_oid_orig);
+
+				if (MACRO_EXPAND_NO == expand_macros)
+					break;
+
+				substitute_simple_macros_unmasked(NULL, NULL, NULL, NULL, &items[i].host.hostid, NULL,
+						NULL, NULL, NULL, &items[i].snmp_community, MACRO_TYPE_COMMON, NULL, 0);
+				if (SUCCEED != substitute_key_macros(&items[i].snmp_oid, &items[i].host.hostid,
+						NULL, NULL, NULL, MACRO_TYPE_SNMP_OID, error, sizeof(error)))
+				{
+					SET_MSG_RESULT(&results[i], zbx_strdup(NULL, error));
+					errcodes[i] = CONFIG_ERROR;
+					continue;
+				}
+				break;
+			case ITEM_TYPE_SSH:
+				ZBX_STRDUP(items[i].publickey, items[i].publickey_orig);
+				ZBX_STRDUP(items[i].privatekey, items[i].privatekey_orig);
+
+				if (MACRO_EXPAND_YES == expand_macros)
+				{
+					substitute_simple_macros(NULL, NULL, NULL, NULL, &items[i].host.hostid, NULL,
+							NULL, NULL, NULL, &items[i].publickey, MACRO_TYPE_COMMON, NULL,
+							0);
+					substitute_simple_macros(NULL, NULL, NULL, NULL, &items[i].host.hostid, NULL,
+							NULL, NULL, NULL, &items[i].privatekey, MACRO_TYPE_COMMON, NULL,
+							0);
+				}
+				ZBX_FALLTHROUGH;
+			case ITEM_TYPE_TELNET:
+			case ITEM_TYPE_DB_MONITOR:
+				if (MACRO_EXPAND_YES == expand_macros)
+				{
+					substitute_simple_macros_unmasked(NULL, NULL, NULL, NULL, NULL, NULL, &items[i],
+							NULL, NULL, &items[i].params, MACRO_TYPE_PARAMS_FIELD, NULL, 0);
+				}
+				ZBX_FALLTHROUGH;
+			case ITEM_TYPE_SIMPLE:
+				items[i].username = zbx_strdup(items[i].username, items[i].username_orig);
+				items[i].password = zbx_strdup(items[i].password, items[i].password_orig);
+
+				if (MACRO_EXPAND_NO == expand_macros)
+					break;
+
+				substitute_simple_macros_unmasked(NULL, NULL, NULL, NULL, &items[i].host.hostid, NULL,
+						NULL, NULL, NULL, &items[i].username, MACRO_TYPE_COMMON, NULL, 0);
+				substitute_simple_macros_unmasked(NULL, NULL, NULL, NULL, &items[i].host.hostid, NULL,
+						NULL, NULL, NULL, &items[i].password, MACRO_TYPE_COMMON, NULL, 0);
+				break;
+			case ITEM_TYPE_JMX:
+				items[i].username = zbx_strdup(items[i].username, items[i].username_orig);
+				items[i].password = zbx_strdup(items[i].password, items[i].password_orig);
+				items[i].jmx_endpoint = zbx_strdup(items[i].jmx_endpoint, items[i].jmx_endpoint_orig);
+
+				if (MACRO_EXPAND_NO == expand_macros)
+					break;
+
+				substitute_simple_macros_unmasked(NULL, NULL, NULL, NULL, &items[i].host.hostid, NULL,
+						NULL, NULL, NULL, &items[i].username, MACRO_TYPE_COMMON, NULL, 0);
+				substitute_simple_macros_unmasked(NULL, NULL, NULL, NULL, &items[i].host.hostid, NULL,
+						NULL, NULL, NULL, &items[i].password, MACRO_TYPE_COMMON, NULL, 0);
+				substitute_simple_macros(NULL, NULL, NULL, NULL, NULL, NULL, &items[i],
+						NULL, NULL, &items[i].jmx_endpoint, MACRO_TYPE_JMX_ENDPOINT, NULL, 0);
+				break;
+			case ITEM_TYPE_HTTPAGENT:
+				ZBX_STRDUP(items[i].timeout, items[i].timeout_orig);
+				ZBX_STRDUP(items[i].url, items[i].url_orig);
+				ZBX_STRDUP(items[i].status_codes, items[i].status_codes_orig);
+				ZBX_STRDUP(items[i].http_proxy, items[i].http_proxy_orig);
+				ZBX_STRDUP(items[i].ssl_cert_file, items[i].ssl_cert_file_orig);
+				ZBX_STRDUP(items[i].ssl_key_file, items[i].ssl_key_file_orig);
+				ZBX_STRDUP(items[i].ssl_key_password, items[i].ssl_key_password_orig);
+				ZBX_STRDUP(items[i].username, items[i].username_orig);
+				ZBX_STRDUP(items[i].password, items[i].password_orig);
+
+				if (MACRO_EXPAND_YES == expand_macros)
+				{
+					substitute_simple_macros(NULL, NULL, NULL, NULL, &items[i].host.hostid, NULL,
+							NULL, NULL, NULL, &items[i].timeout, MACRO_TYPE_COMMON, NULL,
+							0);
+					substitute_simple_macros_unmasked(NULL, NULL, NULL,NULL, NULL, &items[i].host,
+							&items[i], NULL, NULL, &items[i].url, MACRO_TYPE_HTTP_RAW, NULL,
+							0);
+				}
+
+				if (SUCCEED != zbx_http_punycode_encode_url(&items[i].url))
+				{
+					SET_MSG_RESULT(&results[i], zbx_strdup(NULL, "Cannot encode URL into punycode"));
+					errcodes[i] = CONFIG_ERROR;
+					continue;
+				}
+
+				if (FAIL == parse_query_fields(&items[i], &items[i].query_fields, expand_macros))
+				{
+					SET_MSG_RESULT(&results[i], zbx_strdup(NULL, "Invalid query fields"));
+					errcodes[i] = CONFIG_ERROR;
+					continue;
+				}
+
+				if (MACRO_EXPAND_NO == expand_macros)
+					break;
+
+				switch (items[i].post_type)
+				{
+					case ZBX_POSTTYPE_XML:
+						if (SUCCEED != substitute_macros_xml_unmasked(&items[i].posts, &items[i],
+								NULL, NULL, error, sizeof(error)))
+						{
+							SET_MSG_RESULT(&results[i], zbx_dsprintf(NULL, "%s.", error));
+							errcodes[i] = CONFIG_ERROR;
+							continue;
+						}
+						break;
+					case ZBX_POSTTYPE_JSON:
+						substitute_simple_macros_unmasked(NULL, NULL, NULL,NULL, NULL,
+								&items[i].host, &items[i], NULL, NULL, &items[i].posts,
+								MACRO_TYPE_HTTP_JSON, NULL, 0);
+						break;
+					default:
+						substitute_simple_macros_unmasked(NULL, NULL, NULL,NULL, NULL,
+								&items[i].host, &items[i], NULL, NULL, &items[i].posts,
+								MACRO_TYPE_HTTP_RAW, NULL, 0);
+						break;
+				}
+
+				substitute_simple_macros_unmasked(NULL, NULL, NULL,NULL, NULL, &items[i].host,
+						&items[i], NULL, NULL, &items[i].headers, MACRO_TYPE_HTTP_RAW, NULL, 0);
+				substitute_simple_macros(NULL, NULL, NULL, NULL, &items[i].host.hostid, NULL,
+						NULL, NULL, NULL, &items[i].status_codes, MACRO_TYPE_COMMON, NULL, 0);
+				substitute_simple_macros(NULL, NULL, NULL, NULL, &items[i].host.hostid, NULL,
+						NULL, NULL, NULL, &items[i].http_proxy, MACRO_TYPE_COMMON, NULL, 0);
+				substitute_simple_macros(NULL, NULL, NULL,NULL, NULL, &items[i].host, &items[i], NULL,
+						NULL, &items[i].ssl_cert_file, MACRO_TYPE_HTTP_RAW, NULL, 0);
+				substitute_simple_macros(NULL, NULL, NULL,NULL, NULL, &items[i].host, &items[i], NULL,
+						NULL, &items[i].ssl_key_file, MACRO_TYPE_HTTP_RAW, NULL, 0);
+				substitute_simple_macros_unmasked(NULL, NULL, NULL, NULL, &items[i].host.hostid, NULL,
+						NULL, NULL, NULL, &items[i].ssl_key_password, MACRO_TYPE_COMMON, NULL,
+						0);
+				substitute_simple_macros_unmasked(NULL, NULL, NULL, NULL, &items[i].host.hostid, NULL,
+						NULL, NULL, NULL, &items[i].username, MACRO_TYPE_COMMON, NULL, 0);
+				substitute_simple_macros_unmasked(NULL, NULL, NULL, NULL, &items[i].host.hostid, NULL,
+						NULL, NULL, NULL, &items[i].password, MACRO_TYPE_COMMON, NULL, 0);
+				break;
+		}
+	}
+
+	zbx_free(port);
+}
+
+void	zbx_check_items(DC_ITEM *items, int *errcodes, int num, AGENT_RESULT *results, zbx_vector_ptr_t *add_results)
+{
+	if (ITEM_TYPE_SNMP == items[0].type)
+	{
+#ifndef HAVE_NETSNMP
+		int	i;
+
+		for (i = 0; i < num; i++)
+		{
+			if (SUCCEED != errcodes[i])
+				continue;
+
+			SET_MSG_RESULT(&results[i], zbx_strdup(NULL, "Support for SNMP checks was not compiled in."));
+			errcodes[i] = CONFIG_ERROR;
+		}
+#else
+		/* SNMP checks use their own timeouts */
+		get_values_snmp(items, results, errcodes, num);
+#endif
+	}
+	else if (ITEM_TYPE_JMX == items[0].type)
+	{
+		zbx_alarm_on(CONFIG_TIMEOUT);
+		get_values_java(ZBX_JAVA_GATEWAY_REQUEST_JMX, items, results, errcodes, num);
+		zbx_alarm_off();
+	}
+	else if (1 == num)
+	{
+		if (SUCCEED == errcodes[0])
+			errcodes[0] = get_value(&items[0], &results[0], add_results);
+	}
+	else
+		THIS_SHOULD_NEVER_HAPPEN;
+}
+
+void	zbx_clean_items(DC_ITEM *items, int num, AGENT_RESULT *results)
+{
+	int	i;
+
+	for (i = 0; i < num; i++)
+	{
+		zbx_free(items[i].key);
+
+		switch (items[i].type)
+		{
+			case ITEM_TYPE_SNMP:
+				if (ZBX_IF_SNMP_VERSION_3 == items[i].snmp_version)
+				{
+					zbx_free(items[i].snmpv3_securityname);
+					zbx_free(items[i].snmpv3_authpassphrase);
+					zbx_free(items[i].snmpv3_privpassphrase);
+					zbx_free(items[i].snmpv3_contextname);
+				}
+
+				zbx_free(items[i].snmp_community);
+				zbx_free(items[i].snmp_oid);
+				break;
+			case ITEM_TYPE_HTTPAGENT:
+				zbx_free(items[i].timeout);
+				zbx_free(items[i].url);
+				zbx_free(items[i].query_fields);
+				zbx_free(items[i].status_codes);
+				zbx_free(items[i].http_proxy);
+				zbx_free(items[i].ssl_cert_file);
+				zbx_free(items[i].ssl_key_file);
+				zbx_free(items[i].ssl_key_password);
+				zbx_free(items[i].username);
+				zbx_free(items[i].password);
+				break;
+			case ITEM_TYPE_SSH:
+				zbx_free(items[i].publickey);
+				zbx_free(items[i].privatekey);
+				ZBX_FALLTHROUGH;
+			case ITEM_TYPE_TELNET:
+			case ITEM_TYPE_DB_MONITOR:
+			case ITEM_TYPE_SIMPLE:
+				zbx_free(items[i].username);
+				zbx_free(items[i].password);
+				break;
+			case ITEM_TYPE_JMX:
+				zbx_free(items[i].username);
+				zbx_free(items[i].password);
+				zbx_free(items[i].jmx_endpoint);
+				break;
+		}
+
+		free_result(&results[i]);
+	}
+}
+
 /******************************************************************************
  *                                                                            *
  * Function: get_values                                                       *
@@ -501,16 +815,14 @@ static int	parse_query_fields(const DC_ITEM *item, char **query_fields)
  ******************************************************************************/
 static int	get_values(unsigned char poller_type, int *nextcheck)
 {
-	const char		*__function_name = "get_values";
 	DC_ITEM			items[MAX_POLLER_ITEMS];
 	AGENT_RESULT		results[MAX_POLLER_ITEMS];
 	int			errcodes[MAX_POLLER_ITEMS];
 	zbx_timespec_t		timespec;
-	char			*port = NULL, error[ITEM_ERROR_LEN_MAX];
 	int			i, num, last_available = HOST_AVAILABLE_UNKNOWN;
 	zbx_vector_ptr_t	add_results;
 
-	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __function_name);
+	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __func__);
 
 	num = DCconfig_get_poller_items(poller_type, items);
 
@@ -520,219 +832,10 @@ static int	get_values(unsigned char poller_type, int *nextcheck)
 		goto exit;
 	}
 
-	/* prepare items */
-	for (i = 0; i < num; i++)
-	{
-		init_result(&results[i]);
-		errcodes[i] = SUCCEED;
-
-		ZBX_STRDUP(items[i].key, items[i].key_orig);
-		if (SUCCEED != substitute_key_macros(&items[i].key, NULL, &items[i], NULL,
-				MACRO_TYPE_ITEM_KEY, error, sizeof(error)))
-		{
-			SET_MSG_RESULT(&results[i], zbx_strdup(NULL, error));
-			errcodes[i] = CONFIG_ERROR;
-			continue;
-		}
-
-		switch (items[i].type)
-		{
-			case ITEM_TYPE_ZABBIX:
-			case ITEM_TYPE_SNMPv1:
-			case ITEM_TYPE_SNMPv2c:
-			case ITEM_TYPE_SNMPv3:
-			case ITEM_TYPE_JMX:
-				ZBX_STRDUP(port, items[i].interface.port_orig);
-				substitute_simple_macros(NULL, NULL, NULL, NULL, &items[i].host.hostid, NULL,
-						NULL, NULL, NULL, &port, MACRO_TYPE_COMMON, NULL, 0);
-				if (FAIL == is_ushort(port, &items[i].interface.port))
-				{
-					SET_MSG_RESULT(&results[i], zbx_dsprintf(NULL, "Invalid port number [%s]",
-								items[i].interface.port_orig));
-					errcodes[i] = CONFIG_ERROR;
-					continue;
-				}
-				break;
-		}
-
-		switch (items[i].type)
-		{
-			case ITEM_TYPE_SNMPv3:
-				ZBX_STRDUP(items[i].snmpv3_securityname, items[i].snmpv3_securityname_orig);
-				ZBX_STRDUP(items[i].snmpv3_authpassphrase, items[i].snmpv3_authpassphrase_orig);
-				ZBX_STRDUP(items[i].snmpv3_privpassphrase, items[i].snmpv3_privpassphrase_orig);
-				ZBX_STRDUP(items[i].snmpv3_contextname, items[i].snmpv3_contextname_orig);
-
-				substitute_simple_macros(NULL, NULL, NULL, NULL, &items[i].host.hostid, NULL,
-						NULL, NULL, NULL, &items[i].snmpv3_securityname,
-						MACRO_TYPE_COMMON, NULL, 0);
-				substitute_simple_macros(NULL, NULL, NULL, NULL, &items[i].host.hostid, NULL,
-						NULL, NULL, NULL, &items[i].snmpv3_authpassphrase,
-						MACRO_TYPE_COMMON, NULL, 0);
-				substitute_simple_macros(NULL, NULL, NULL, NULL, &items[i].host.hostid, NULL,
-						NULL, NULL, NULL, &items[i].snmpv3_privpassphrase,
-						MACRO_TYPE_COMMON, NULL, 0);
-				substitute_simple_macros(NULL, NULL, NULL, NULL, &items[i].host.hostid, NULL,
-						NULL, NULL, NULL, &items[i].snmpv3_contextname,
-						MACRO_TYPE_COMMON, NULL, 0);
-				ZBX_FALLTHROUGH;
-			case ITEM_TYPE_SNMPv1:
-			case ITEM_TYPE_SNMPv2c:
-				ZBX_STRDUP(items[i].snmp_community, items[i].snmp_community_orig);
-				ZBX_STRDUP(items[i].snmp_oid, items[i].snmp_oid_orig);
-
-				substitute_simple_macros(NULL, NULL, NULL, NULL, &items[i].host.hostid, NULL,
-						NULL, NULL, NULL, &items[i].snmp_community, MACRO_TYPE_COMMON, NULL, 0);
-				if (SUCCEED != substitute_key_macros(&items[i].snmp_oid, &items[i].host.hostid, NULL,
-						NULL, MACRO_TYPE_SNMP_OID, error, sizeof(error)))
-				{
-					SET_MSG_RESULT(&results[i], zbx_strdup(NULL, error));
-					errcodes[i] = CONFIG_ERROR;
-					continue;
-				}
-				break;
-			case ITEM_TYPE_SSH:
-				ZBX_STRDUP(items[i].publickey, items[i].publickey_orig);
-				ZBX_STRDUP(items[i].privatekey, items[i].privatekey_orig);
-
-				substitute_simple_macros(NULL, NULL, NULL, NULL, &items[i].host.hostid, NULL,
-						NULL, NULL, NULL, &items[i].publickey, MACRO_TYPE_COMMON, NULL, 0);
-				substitute_simple_macros(NULL, NULL, NULL, NULL, &items[i].host.hostid, NULL,
-						NULL, NULL, NULL, &items[i].privatekey, MACRO_TYPE_COMMON, NULL, 0);
-				ZBX_FALLTHROUGH;
-			case ITEM_TYPE_TELNET:
-			case ITEM_TYPE_DB_MONITOR:
-				substitute_simple_macros(NULL, NULL, NULL, NULL, NULL, NULL, &items[i],
-						NULL, NULL, &items[i].params, MACRO_TYPE_PARAMS_FIELD, NULL, 0);
-				ZBX_FALLTHROUGH;
-			case ITEM_TYPE_SIMPLE:
-				items[i].username = zbx_strdup(items[i].username, items[i].username_orig);
-				items[i].password = zbx_strdup(items[i].password, items[i].password_orig);
-
-				substitute_simple_macros(NULL, NULL, NULL, NULL, &items[i].host.hostid, NULL,
-						NULL, NULL, NULL, &items[i].username, MACRO_TYPE_COMMON, NULL, 0);
-				substitute_simple_macros(NULL, NULL, NULL, NULL, &items[i].host.hostid, NULL,
-						NULL, NULL, NULL, &items[i].password, MACRO_TYPE_COMMON, NULL, 0);
-				break;
-			case ITEM_TYPE_JMX:
-				items[i].username = zbx_strdup(items[i].username, items[i].username_orig);
-				items[i].password = zbx_strdup(items[i].password, items[i].password_orig);
-				items[i].jmx_endpoint = zbx_strdup(items[i].jmx_endpoint, items[i].jmx_endpoint_orig);
-
-				substitute_simple_macros(NULL, NULL, NULL, NULL, &items[i].host.hostid, NULL,
-						NULL, NULL, NULL, &items[i].username, MACRO_TYPE_COMMON, NULL, 0);
-				substitute_simple_macros(NULL, NULL, NULL, NULL, &items[i].host.hostid, NULL,
-						NULL, NULL, NULL, &items[i].password, MACRO_TYPE_COMMON, NULL, 0);
-				substitute_simple_macros(NULL, NULL, NULL, NULL, NULL, NULL, &items[i],
-						NULL, NULL, &items[i].jmx_endpoint, MACRO_TYPE_JMX_ENDPOINT, NULL, 0);
-				break;
-			case ITEM_TYPE_HTTPAGENT:
-				ZBX_STRDUP(items[i].timeout, items[i].timeout_orig);
-				ZBX_STRDUP(items[i].url, items[i].url_orig);
-				ZBX_STRDUP(items[i].status_codes, items[i].status_codes_orig);
-				ZBX_STRDUP(items[i].http_proxy, items[i].http_proxy_orig);
-				ZBX_STRDUP(items[i].ssl_cert_file, items[i].ssl_cert_file_orig);
-				ZBX_STRDUP(items[i].ssl_key_file, items[i].ssl_key_file_orig);
-				ZBX_STRDUP(items[i].ssl_key_password, items[i].ssl_key_password_orig);
-				ZBX_STRDUP(items[i].username, items[i].username_orig);
-				ZBX_STRDUP(items[i].password, items[i].password_orig);
-
-				substitute_simple_macros(NULL, NULL, NULL, NULL, &items[i].host.hostid, NULL,
-						NULL, NULL, NULL, &items[i].timeout, MACRO_TYPE_COMMON, NULL, 0);
-				substitute_simple_macros(NULL, NULL, NULL,NULL, NULL, &items[i].host, &items[i], NULL,
-						NULL, &items[i].url, MACRO_TYPE_HTTP_RAW, NULL, 0);
-
-				if (SUCCEED != zbx_http_punycode_encode_url(&items[i].url))
-				{
-					SET_MSG_RESULT(&results[i], zbx_strdup(NULL, "Cannot encode URL into punycode"));
-					errcodes[i] = CONFIG_ERROR;
-					continue;
-				}
-
-				if (FAIL == parse_query_fields(&items[i], &items[i].query_fields))
-				{
-					SET_MSG_RESULT(&results[i], zbx_strdup(NULL, "Invalid query fields"));
-					errcodes[i] = CONFIG_ERROR;
-					continue;
-				}
-
-				switch (items[i].post_type)
-				{
-					case ZBX_POSTTYPE_XML:
-						if (SUCCEED != substitute_macros_xml(&items[i].posts, &items[i], NULL,
-								error, sizeof(error)))
-						{
-							SET_MSG_RESULT(&results[i], zbx_dsprintf(NULL, "%s.", error));
-							errcodes[i] = CONFIG_ERROR;
-							continue;
-						}
-						break;
-					case ZBX_POSTTYPE_JSON:
-						substitute_simple_macros(NULL, NULL, NULL,NULL, NULL, &items[i].host,
-								&items[i], NULL, NULL, &items[i].posts,
-								MACRO_TYPE_HTTP_JSON, NULL, 0);
-						break;
-					default:
-						substitute_simple_macros(NULL, NULL, NULL,NULL, NULL, &items[i].host,
-								&items[i], NULL, NULL, &items[i].posts,
-								MACRO_TYPE_HTTP_RAW, NULL, 0);
-						break;
-				}
-
-				substitute_simple_macros(NULL, NULL, NULL,NULL, NULL, &items[i].host, &items[i], NULL,
-						NULL, &items[i].headers, MACRO_TYPE_HTTP_RAW, NULL, 0);
-				substitute_simple_macros(NULL, NULL, NULL, NULL, &items[i].host.hostid, NULL,
-						NULL, NULL, NULL, &items[i].status_codes, MACRO_TYPE_COMMON, NULL, 0);
-				substitute_simple_macros(NULL, NULL, NULL, NULL, &items[i].host.hostid, NULL,
-						NULL, NULL, NULL, &items[i].http_proxy, MACRO_TYPE_COMMON, NULL, 0);
-				substitute_simple_macros(NULL, NULL, NULL,NULL, NULL, &items[i].host, &items[i], NULL,
-						NULL, &items[i].ssl_cert_file, MACRO_TYPE_HTTP_RAW, NULL, 0);
-				substitute_simple_macros(NULL, NULL, NULL,NULL, NULL, &items[i].host, &items[i], NULL,
-						NULL, &items[i].ssl_key_file, MACRO_TYPE_HTTP_RAW, NULL, 0);
-				substitute_simple_macros(NULL, NULL, NULL, NULL, &items[i].host.hostid, NULL, NULL,
-						NULL, NULL, &items[i].ssl_key_password, MACRO_TYPE_COMMON, NULL, 0);
-				substitute_simple_macros(NULL, NULL, NULL, NULL, &items[i].host.hostid, NULL,
-						NULL, NULL, NULL, &items[i].username, MACRO_TYPE_COMMON, NULL, 0);
-				substitute_simple_macros(NULL, NULL, NULL, NULL, &items[i].host.hostid, NULL,
-						NULL, NULL, NULL, &items[i].password, MACRO_TYPE_COMMON, NULL, 0);
-				break;
-		}
-	}
-
-	zbx_free(port);
-
 	zbx_vector_ptr_create(&add_results);
 
-	/* retrieve item values */
-	if (SUCCEED == is_snmp_type(items[0].type))
-	{
-#ifdef HAVE_NETSNMP
-		/* SNMP checks use their own timeouts */
-		get_values_snmp(items, results, errcodes, num);
-#else
-		for (i = 0; i < num; i++)
-		{
-			if (SUCCEED != errcodes[i])
-				continue;
-
-			SET_MSG_RESULT(&results[i], zbx_strdup(NULL, "Support for SNMP checks was not compiled in."));
-			errcodes[i] = CONFIG_ERROR;
-		}
-#endif
-	}
-	else if (ITEM_TYPE_JMX == items[0].type)
-	{
-		zbx_alarm_on(CONFIG_TIMEOUT);
-		get_values_java(ZBX_JAVA_GATEWAY_REQUEST_JMX, items, results, errcodes, num);
-		zbx_alarm_off();
-	}
-	else if (1 == num)
-	{
-		if (SUCCEED == errcodes[0])
-			errcodes[0] = get_value(&items[0], &results[0], &add_results);
-	}
-	else
-		THIS_SHOULD_NEVER_HAPPEN;
+	zbx_prepare_items(items, errcodes, num, results, MACRO_EXPAND_YES);
+	zbx_check_items(items, errcodes, num, results, &add_results);
 
 	zbx_timespec(&timespec);
 
@@ -819,65 +922,29 @@ static int	get_values(unsigned char poller_type, int *nextcheck)
 
 		DCpoller_requeue_items(&items[i].itemid, &items[i].state, &timespec.sec, &errcodes[i], 1, poller_type,
 				nextcheck);
-
-		zbx_free(items[i].key);
-
-		switch (items[i].type)
-		{
-			case ITEM_TYPE_SNMPv3:
-				zbx_free(items[i].snmpv3_securityname);
-				zbx_free(items[i].snmpv3_authpassphrase);
-				zbx_free(items[i].snmpv3_privpassphrase);
-				zbx_free(items[i].snmpv3_contextname);
-				ZBX_FALLTHROUGH;
-			case ITEM_TYPE_SNMPv1:
-			case ITEM_TYPE_SNMPv2c:
-				zbx_free(items[i].snmp_community);
-				zbx_free(items[i].snmp_oid);
-				break;
-			case ITEM_TYPE_HTTPAGENT:
-				zbx_free(items[i].timeout);
-				zbx_free(items[i].url);
-				zbx_free(items[i].query_fields);
-				zbx_free(items[i].status_codes);
-				zbx_free(items[i].http_proxy);
-				zbx_free(items[i].ssl_cert_file);
-				zbx_free(items[i].ssl_key_file);
-				zbx_free(items[i].ssl_key_password);
-				zbx_free(items[i].username);
-				zbx_free(items[i].password);
-				break;
-			case ITEM_TYPE_SSH:
-				zbx_free(items[i].publickey);
-				zbx_free(items[i].privatekey);
-				ZBX_FALLTHROUGH;
-			case ITEM_TYPE_TELNET:
-			case ITEM_TYPE_DB_MONITOR:
-			case ITEM_TYPE_SIMPLE:
-				zbx_free(items[i].username);
-				zbx_free(items[i].password);
-				break;
-			case ITEM_TYPE_JMX:
-				zbx_free(items[i].username);
-				zbx_free(items[i].password);
-				zbx_free(items[i].jmx_endpoint);
-				break;
-		}
-
-		free_result(&results[i]);
 	}
 
 	zbx_preprocessor_flush();
-	zbx_vector_ptr_clear_ext(&add_results, (zbx_mem_free_func_t)free_result_ptr);
-	zbx_vector_ptr_destroy(&add_results);
-
+	zbx_clean_items(items, num, results);
 	DCconfig_clean_items(items, NULL, num);
+	zbx_vector_ptr_clear_ext(&add_results, (zbx_mem_free_func_t)zbx_free_result_ptr);
+	zbx_vector_ptr_destroy(&add_results);
 exit:
-	zabbix_log(LOG_LEVEL_DEBUG, "End of %s():%d", __function_name, num);
+	zabbix_log(LOG_LEVEL_DEBUG, "End of %s():%d", __func__, num);
 
 	return num;
 }
 
+static void	zbx_poller_sigusr_handler(int flags)
+{
+#ifdef HAVE_NETSNMP
+	if (ZBX_RTC_SNMP_CACHE_RELOAD == ZBX_RTC_GET_MSG(flags))
+		snmp_cache_reload_requested = 1;
+#else
+	ZBX_UNUSED(flags);
+#endif
+}
+
 ZBX_THREAD_ENTRY(poller_thread, args)
 {
 	int		nextcheck, sleeptime = -1, processed = 0, old_processed = 0;
@@ -903,7 +970,7 @@ ZBX_THREAD_ENTRY(poller_thread, args)
 		zbx_init_snmp();
 #endif
 
-#if defined(HAVE_POLARSSL) || defined(HAVE_GNUTLS) || defined(HAVE_OPENSSL)
+#if defined(HAVE_GNUTLS) || defined(HAVE_OPENSSL)
 	zbx_tls_init_child();
 #endif
 	zbx_setproctitle("%s #%d [connecting to the database]", get_process_type_string(process_type), process_num);
@@ -911,11 +978,21 @@ ZBX_THREAD_ENTRY(poller_thread, args)
 
 	DBconnect(ZBX_DB_CONNECT_NORMAL);
 
+	zbx_set_sigusr_handler(zbx_poller_sigusr_handler);
+
 	while (ZBX_IS_RUNNING())
 	{
 		sec = zbx_time();
 		zbx_update_env(sec);
 
+#ifdef HAVE_NETSNMP
+		if ((ZBX_POLLER_TYPE_NORMAL == poller_type || ZBX_POLLER_TYPE_UNREACHABLE == poller_type) &&
+				1 == snmp_cache_reload_requested)
+		{
+			zbx_clear_cache_snmp();
+			snmp_cache_reload_requested = 0;
+		}
+#endif
 		if (0 != sleeptime)
 		{
 			zbx_setproctitle("%s #%d [got %d values in " ZBX_FS_DBL " sec, getting values]",
